(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://qt.nokia.com/doc/4.0/qpl.html>.
 *)

(** Conversion and compilation of simple bytecode primitives. *)

exception Not_a_call
(** Exception raised when a primitive cannot be converted to a call. *)

val as_mcall : Lambda.primitive -> Macroinstr.expression list -> Macroinstr.expression
(** Tries to convert a primitive into a call, raising [Not_a_call] if such a
   conversion fails. *)

val is_mcall : Lambda.primitive -> bool
(** Tests whether the passed primitive will be converted into a call. *)

val parameters_type : Lambda.primitive -> Macroinstr.boxed
(** Returns the type of parameters the passed primitive waits for its
   compilation. *)

val compile : Lambda.primitive -> (BaristaLibrary.Instruction.t list * int)
(** Compiles a primitive, returning a list of instructions as well as its size. *)
