(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://qt.nokia.com/doc/4.0/qpl.html>.
 *)

open BaristaLibrary
open Utils


type error =
  | Undefined_class of string * (string option)
  | Undefined_primitive of string

exception Error of error

let stdlib_classes = [
  "fr.x9c.cadmium.primitives.stdlib.Alloc" ;
  "fr.x9c.cadmium.primitives.stdlib.Array" ;
  "fr.x9c.cadmium.primitives.stdlib.BackTrace" ;
  "fr.x9c.cadmium.primitives.stdlib.Callback" ;
  "fr.x9c.cadmium.primitives.stdlib.Compare" ;
  "fr.x9c.cadmium.primitives.stdlib.Dynlink" ;
  "fr.x9c.cadmium.primitives.stdlib.Extern" ;
  "fr.x9c.cadmium.primitives.stdlib.Finalise" ;
  "fr.x9c.cadmium.primitives.stdlib.Floats" ;
  "fr.x9c.cadmium.primitives.stdlib.Gc_ctrl" ;
  "fr.x9c.cadmium.primitives.stdlib.Hash" ;
  "fr.x9c.cadmium.primitives.stdlib.Intern" ;
  "fr.x9c.cadmium.primitives.stdlib.Ints" ;
  "fr.x9c.cadmium.primitives.stdlib.Io" ;
  "fr.x9c.cadmium.primitives.stdlib.Lexing" ;
  "fr.x9c.cadmium.primitives.stdlib.Md5" ;
  "fr.x9c.cadmium.primitives.stdlib.Meta" ;
  "fr.x9c.cadmium.primitives.stdlib.NatDynlink" ;
  "fr.x9c.cadmium.primitives.stdlib.Obj" ;
  "fr.x9c.cadmium.primitives.stdlib.Parsing" ;
  "fr.x9c.cadmium.primitives.stdlib.Signals" ;
  "fr.x9c.cadmium.primitives.stdlib.Stacks" ;
  "fr.x9c.cadmium.primitives.stdlib.Str" ;
  "fr.x9c.cadmium.primitives.stdlib.Sys" ;
  "fr.x9c.cadmium.primitives.stdlib.Terminfo" ;
  "fr.x9c.cadmium.primitives.stdlib.Weak"
]

let cadmium_classes = [
  "fr.x9c.cadmium.primitives.cadmium.Applets" ;
  "fr.x9c.cadmium.primitives.cadmium.Arrays" ;
  "fr.x9c.cadmium.primitives.cadmium.Cadmium" ;
  "fr.x9c.cadmium.primitives.cadmium.Classes" ;
  "fr.x9c.cadmium.primitives.cadmium.Constructors" ;
  "fr.x9c.cadmium.primitives.cadmium.CreateInterface" ;
  "fr.x9c.cadmium.primitives.cadmium.Fields" ;
  "fr.x9c.cadmium.primitives.cadmium.Methods" ;
  "fr.x9c.cadmium.primitives.cadmium.Objects" ;
  "fr.x9c.cadmium.primitives.cadmium.Proxies" ;
  "fr.x9c.cadmium.primitives.cadmium.Runtime"
]

let cadmium_jdbc_classes = [
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Arrays";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Blobs";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.CallableStatements";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Clobs";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Connections";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.DatabaseMetaDatas";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.DateBase";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Dates";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.DriverManagers";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Drivers";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.ParameterMetaDatas";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.PreparedStatements";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Refs";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.ResultSetMetaDatas";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.ResultSets";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.RowIds";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.SQLXMLs";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Savepoints";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Statements";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Structs";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Times";
  "fr.x9c.cadmium.primitives.cadmiumjdbc.Timestamps"
]

let cadmium_math_classes = [
  "fr.x9c.cadmium.primitives.cadmiummath.BigDecimals";
  "fr.x9c.cadmium.primitives.cadmiummath.BigIntegers";
  "fr.x9c.cadmium.primitives.cadmiummath.Floats";
  "fr.x9c.cadmium.primitives.cadmiummath.Randoms";
  "fr.x9c.cadmium.primitives.cadmiummath.StrictFloats"
]

let cadmium_servlet_classes = [
  "fr.x9c.cadmium.primitives.cadmiumservlet.Servlets"
]

let cadmium_swixml_classes = [
  "fr.x9c.cadmium.primitives.cadmiumswixml.SwiXml"
]

let cadmium_xml_classes = [
  "fr.x9c.cadmium.primitives.cadmiumxml.Documents" ;
  "fr.x9c.cadmium.primitives.cadmiumxml.SAX" ;
  "fr.x9c.cadmium.primitives.cadmiumxml.Schemas" ;
  "fr.x9c.cadmium.primitives.cadmiumxml.Transformers" ;
  "fr.x9c.cadmium.primitives.cadmiumxml.Trees" ;
  "fr.x9c.cadmium.primitives.cadmiumxml.XPath"
]

let bigarray_classes = [
  "fr.x9c.cadmium.primitives.bigarray.BigArray"
]

let dbm_classes = [
  "fr.x9c.cadmium.primitives.dbm.Cldbm"
]

let graph_classes = [
  "fr.x9c.cadmium.primitives.graph.Color" ;
  "fr.x9c.cadmium.primitives.graph.Draw" ;
  "fr.x9c.cadmium.primitives.graph.DumpImage" ;
  "fr.x9c.cadmium.primitives.graph.Events" ;
  "fr.x9c.cadmium.primitives.graph.Fill" ;
  "fr.x9c.cadmium.primitives.graph.Image" ;
  "fr.x9c.cadmium.primitives.graph.MakeImage" ;
  "fr.x9c.cadmium.primitives.graph.Open" ;
  "fr.x9c.cadmium.primitives.graph.PointCol" ;
  "fr.x9c.cadmium.primitives.graph.Sound" ;
  "fr.x9c.cadmium.primitives.graph.SubWindow" ;
  "fr.x9c.cadmium.primitives.graph.Text"
]

let labltk_classes = [
  "fr.x9c.cadmium.primitives.labltk.CltkCaml" ;
  "fr.x9c.cadmium.primitives.labltk.CltkEval" ;
  "fr.x9c.cadmium.primitives.labltk.CltkEvent" ;
  "fr.x9c.cadmium.primitives.labltk.CltkFile" ;
  "fr.x9c.cadmium.primitives.labltk.CltkImg" ;
  "fr.x9c.cadmium.primitives.labltk.CltkMain" ;
  "fr.x9c.cadmium.primitives.labltk.CltkMisc" ;
  "fr.x9c.cadmium.primitives.labltk.CltkTimer" ;
  "fr.x9c.cadmium.primitives.labltk.CltkVar" ;
  "fr.x9c.cadmium.primitives.labltk.CltkWait"
]

let num_classes = [
  "fr.x9c.cadmium.primitives.num.Nat"
]

let str_classes = [
  "fr.x9c.cadmium.primitives.str.Str"
]

let systhreads_classes = [
  "fr.x9c.cadmium.primitives.systhreads.Threads"
]

let threads_classes = [
  "fr.x9c.cadmium.primitives.threads.Threads"
]

let unix_classes = [
  "fr.x9c.cadmium.primitives.unix.Accept" ;
  "fr.x9c.cadmium.primitives.unix.Access" ;
  "fr.x9c.cadmium.primitives.unix.Addrofstr" ;
  "fr.x9c.cadmium.primitives.unix.Alarm" ;
  "fr.x9c.cadmium.primitives.unix.Bind" ;
  "fr.x9c.cadmium.primitives.unix.Chdir" ;
  "fr.x9c.cadmium.primitives.unix.Chmod" ;
  "fr.x9c.cadmium.primitives.unix.Chown" ;
  "fr.x9c.cadmium.primitives.unix.Chroot" ;
  "fr.x9c.cadmium.primitives.unix.Close" ;
  "fr.x9c.cadmium.primitives.unix.Closedir" ;
  "fr.x9c.cadmium.primitives.unix.Connect" ;
  "fr.x9c.cadmium.primitives.unix.Dup" ;
  "fr.x9c.cadmium.primitives.unix.Dup2" ;
  "fr.x9c.cadmium.primitives.unix.Envir" ;
  "fr.x9c.cadmium.primitives.unix.Errmsg" ;
  "fr.x9c.cadmium.primitives.unix.Execv" ;
  "fr.x9c.cadmium.primitives.unix.Execve" ;
  "fr.x9c.cadmium.primitives.unix.Execvp" ;
  "fr.x9c.cadmium.primitives.unix.Exit" ;
  "fr.x9c.cadmium.primitives.unix.Fchmod" ;
  "fr.x9c.cadmium.primitives.unix.Fchown" ;
  "fr.x9c.cadmium.primitives.unix.Fcntl" ;
  "fr.x9c.cadmium.primitives.unix.Fork" ;
  "fr.x9c.cadmium.primitives.unix.Ftruncate" ;
  "fr.x9c.cadmium.primitives.unix.Getaddrinfo" ;
  "fr.x9c.cadmium.primitives.unix.Getcwd" ;
  "fr.x9c.cadmium.primitives.unix.Getegid" ;
  "fr.x9c.cadmium.primitives.unix.Geteuid" ;
  "fr.x9c.cadmium.primitives.unix.Getgid" ;
  "fr.x9c.cadmium.primitives.unix.Getgr" ;
  "fr.x9c.cadmium.primitives.unix.Getgroups" ;
  "fr.x9c.cadmium.primitives.unix.Gethost" ;
  "fr.x9c.cadmium.primitives.unix.Gethostname" ;
  "fr.x9c.cadmium.primitives.unix.Getlogin" ;
  "fr.x9c.cadmium.primitives.unix.Getnameinfo" ;
  "fr.x9c.cadmium.primitives.unix.Getpeername" ;
  "fr.x9c.cadmium.primitives.unix.Getpid" ;
  "fr.x9c.cadmium.primitives.unix.Getppid" ;
  "fr.x9c.cadmium.primitives.unix.Getproto" ;
  "fr.x9c.cadmium.primitives.unix.Getpw" ;
  "fr.x9c.cadmium.primitives.unix.Getserv" ;
  "fr.x9c.cadmium.primitives.unix.Getsockname" ;
  "fr.x9c.cadmium.primitives.unix.Gettimeofday" ;
  "fr.x9c.cadmium.primitives.unix.Getuid" ;
  "fr.x9c.cadmium.primitives.unix.Gmtime" ;
  "fr.x9c.cadmium.primitives.unix.Isatty" ;
  "fr.x9c.cadmium.primitives.unix.Itimer" ;
  "fr.x9c.cadmium.primitives.unix.Kill" ;
  "fr.x9c.cadmium.primitives.unix.Link" ;
  "fr.x9c.cadmium.primitives.unix.Listen" ;
  "fr.x9c.cadmium.primitives.unix.Lockf" ;
  "fr.x9c.cadmium.primitives.unix.Lseek" ;
  "fr.x9c.cadmium.primitives.unix.Mkdir" ;
  "fr.x9c.cadmium.primitives.unix.Mkfifo" ;
  "fr.x9c.cadmium.primitives.unix.Nice" ;
  "fr.x9c.cadmium.primitives.unix.Open" ;
  "fr.x9c.cadmium.primitives.unix.Opendir" ;
  "fr.x9c.cadmium.primitives.unix.Pipe" ;
  "fr.x9c.cadmium.primitives.unix.Putenv" ;
  "fr.x9c.cadmium.primitives.unix.Read" ;
  "fr.x9c.cadmium.primitives.unix.Readdir" ;
  "fr.x9c.cadmium.primitives.unix.Readlink" ;
  "fr.x9c.cadmium.primitives.unix.Rename" ;
  "fr.x9c.cadmium.primitives.unix.Rewinddir" ;
  "fr.x9c.cadmium.primitives.unix.Rmdir" ;
  "fr.x9c.cadmium.primitives.unix.Select" ;
  "fr.x9c.cadmium.primitives.unix.Sendrecv" ;
  "fr.x9c.cadmium.primitives.unix.Setgid" ;
  "fr.x9c.cadmium.primitives.unix.Setsid" ;
  "fr.x9c.cadmium.primitives.unix.Setuid" ;
  "fr.x9c.cadmium.primitives.unix.Shutdown" ;
  "fr.x9c.cadmium.primitives.unix.Signals" ;
  "fr.x9c.cadmium.primitives.unix.Sleep" ;
  "fr.x9c.cadmium.primitives.unix.Socket" ;
  "fr.x9c.cadmium.primitives.unix.Socketpair" ;
  "fr.x9c.cadmium.primitives.unix.Sockopt" ;
  "fr.x9c.cadmium.primitives.unix.Stat" ;
  "fr.x9c.cadmium.primitives.unix.Strofaddr" ;
  "fr.x9c.cadmium.primitives.unix.Symlink" ;
  "fr.x9c.cadmium.primitives.unix.Termios" ;
  "fr.x9c.cadmium.primitives.unix.Time" ;
  "fr.x9c.cadmium.primitives.unix.Times" ;
  "fr.x9c.cadmium.primitives.unix.Truncate" ;
  "fr.x9c.cadmium.primitives.unix.Umask" ;
  "fr.x9c.cadmium.primitives.unix.Unlink" ;
  "fr.x9c.cadmium.primitives.unix.Utimes" ;
  "fr.x9c.cadmium.primitives.unix.Wait" ;
  "fr.x9c.cadmium.primitives.unix.Write"
]

let camlzip_classes = [
  "fr.x9c.cadmium.primitives.zip.Zlib"
]

let dbm_classes = [
  "fr.x9c.cadmium.primitives.dbm.Cldbm"
]

let all_classes =
  stdlib_classes
  @ cadmium_classes
  @ cadmium_jdbc_classes
  @ cadmium_math_classes
  @ cadmium_servlet_classes
  @ cadmium_swixml_classes
  @ cadmium_xml_classes
  @ bigarray_classes
  @ dbm_classes
  @ graph_classes
  @ labltk_classes
  @ num_classes
  @ str_classes
  @ systhreads_classes
  @ threads_classes
  @ unix_classes
  @ camlzip_classes
  @ dbm_classes

let builtin = [
  ("fr.x9c.cadmium.primitives.stdlib.Alloc", [
    "caml_alloc_dummy" ;
    "caml_alloc_dummy_float" ;
    "caml_update_dummy"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Array", [
    "caml_array_get_addr" ;
    "caml_array_get_float" ;
    "caml_array_get" ;
    "caml_array_set_addr" ;
    "caml_array_set_float" ;
    "caml_array_set" ;
    "caml_array_unsafe_get_float" ;
    "caml_array_unsafe_get" ;
    "caml_array_unsafe_set_addr" ;
    "caml_array_unsafe_set_float" ;
    "caml_array_unsafe_set" ;
    "caml_make_vect" ;
    "caml_make_array"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.BackTrace", [
    "caml_record_backtrace" ;
    "caml_backtrace_status" ;
    "caml_get_exception_backtrace"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Callback", [
    "caml_register_named_value"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Compare", [
    "caml_compare" ;
    "caml_equal" ;
    "caml_notequal" ;
    "caml_lessthan" ;
    "caml_lessequal" ;
    "caml_greaterthan" ;
    "caml_greaterequal"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Dynlink", [
    "caml_dynlink_open_lib" ;
    "caml_dynlink_close_lib" ;
    "caml_dynlink_lookup_symbol" ;
    "caml_dynlink_add_primitive" ;
    "caml_dynlink_get_current_libs"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Extern", [
    "caml_output_value" ;
    "caml_output_value_to_string" ;
    "caml_output_value_to_buffer"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Finalise", [
    "caml_final_register" ;
    "caml_final_release"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Floats", [
    "caml_format_float" ;
    "caml_float_of_string" ;
    "caml_int_of_float" ;
    "caml_float_of_int" ;
    "caml_neg_float" ;
    "caml_abs_float" ;
    "caml_add_float" ;
    "caml_sub_float" ;
    "caml_mul_float" ;
    "caml_div_float" ;
    "caml_exp_float" ;
    "caml_floor_float" ;
    "caml_fmod_float" ;
    "caml_frexp_float" ;
    "caml_ldexp_float" ;
    "caml_log_float" ;
    "caml_log10_float" ;
    "caml_modf_float" ;
    "caml_sqrt_float" ;
    "caml_power_float" ;
    "caml_sin_float" ;
    "caml_sinh_float" ;
    "caml_cos_float" ;
    "caml_cosh_float" ;
    "caml_tan_float" ;
    "caml_tanh_float" ;
    "caml_asin_float" ;
    "caml_acos_float" ;
    "caml_atan_float" ;
    "caml_atan2_float" ;
    "caml_ceil_float" ;
    "caml_eq_float" ;
    "caml_neq_float" ;
    "caml_le_float" ;
    "caml_lt_float" ;
    "caml_ge_float" ;
    "caml_gt_float" ;
    "caml_float_compare" ;
    "caml_classify_float"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Gc_ctrl", [
    "caml_gc_stat" ;
    "caml_gc_quick_stat" ;
    "caml_gc_counters" ;
    "caml_gc_get" ;
    "caml_gc_set" ;
    "caml_gc_minor" ;
    "caml_gc_major" ;
    "caml_gc_full_major" ;
    "caml_gc_major_slice" ;
    "caml_gc_compaction"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Hash", [
    "caml_hash_univ_param"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Intern", [
    "caml_input_value" ;
    "caml_input_value_from_string" ;
    "caml_marshal_data_size"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Ints", [
    "caml_int_compare" ;
    "caml_int_of_string" ;
    "caml_format_int" ;
    "caml_int32_neg" ;
    "caml_int32_add" ;
    "caml_int32_sub" ;
    "caml_int32_mul" ;
    "caml_int32_div" ;
    "caml_int32_mod" ;
    "caml_int32_and" ;
    "caml_int32_or" ;
    "caml_int32_xor" ;
    "caml_int32_shift_left" ;
    "caml_int32_shift_right" ;
    "caml_int32_shift_right_unsigned" ;
    "caml_int32_of_int" ;
    "caml_int32_to_int" ;
    "caml_int32_of_float" ;
    "caml_int32_to_float" ;
    "caml_int32_compare" ;
    "caml_int32_format" ;
    "caml_int32_of_string" ;
    "caml_int32_bits_of_float" ;
    "caml_int32_float_of_bits" ;
    "caml_int64_neg" ;
    "caml_int64_add" ;
    "caml_int64_sub" ;
    "caml_int64_mul" ;
    "caml_int64_div" ;
    "caml_int64_mod" ;
    "caml_int64_and" ;
    "caml_int64_or" ;
    "caml_int64_xor" ;
    "caml_int64_shift_left" ;
    "caml_int64_shift_right" ;
    "caml_int64_shift_right_unsigned" ;
    "caml_int64_of_int" ;
    "caml_int64_to_int" ;
    "caml_int64_of_float" ;
    "caml_int64_to_float" ;
    "caml_int64_of_int32" ;
    "caml_int64_to_int32" ;
    "caml_int64_of_nativeint" ;
    "caml_int64_to_nativeint" ;
    "caml_int64_compare" ;
    "caml_int64_format" ;
    "caml_int64_of_string" ;
    "caml_int64_bits_of_float" ;
    "caml_int64_float_of_bits" ;
    "caml_nativeint_neg" ;
    "caml_nativeint_add" ;
    "caml_nativeint_sub" ;
    "caml_nativeint_mul" ;
    "caml_nativeint_div" ;
    "caml_nativeint_mod" ;
    "caml_nativeint_and" ;
    "caml_nativeint_or" ;
    "caml_nativeint_xor" ;
    "caml_nativeint_shift_left" ;
    "caml_nativeint_shift_right" ;
    "caml_nativeint_shift_right_unsigned" ;
    "caml_nativeint_of_int" ;
    "caml_nativeint_to_int" ;
    "caml_nativeint_of_float" ;
    "caml_nativeint_to_float" ;
    "caml_nativeint_of_int32" ;
    "caml_nativeint_to_int32" ;
    "caml_nativeint_compare" ;
    "caml_nativeint_format" ;
    "caml_nativeint_of_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Io", [
    "caml_ml_open_descriptor_in" ;
    "caml_ml_open_descriptor_out" ;
    "caml_ml_out_channels_list" ;
    "caml_channel_descriptor" ;
    "caml_ml_close_channel" ;
    "caml_ml_channel_size" ;
    "caml_ml_channel_size_64" ;
    "caml_ml_set_binary_mode" ;
    "caml_ml_flush_partial" ;
    "caml_ml_flush" ;
    "caml_ml_output_char" ;
    "caml_ml_output_int" ;
    "caml_ml_output_partial" ;
    "caml_ml_output" ;
    "caml_ml_seek_out" ;
    "caml_ml_seek_out_64" ;
    "caml_ml_pos_out" ;
    "caml_ml_pos_out_64" ;
    "caml_ml_input_char" ;
    "caml_ml_input_int" ;
    "caml_ml_input" ;
    "caml_ml_seek_in" ;
    "caml_ml_seek_in_64" ;
    "caml_ml_pos_in" ;
    "caml_ml_pos_in_64" ;
    "caml_ml_input_scan_line"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Lexing", [
    "caml_lex_engine" ;
    "caml_new_lex_engine"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Md5", [
    "caml_md5_string" ;
    "caml_md5_chan"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Meta", [
    "caml_get_global_data" ;
    "caml_get_section_table" ;
    "caml_reify_bytecode" ;
    "caml_realloc_global" ;
    "caml_get_current_environment" ;
    "caml_invoke_traced_function"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.NatDynlink", [
    "caml_natdynlink_getmap" ;
    "caml_natdynlink_globals_inited" ;
    "caml_natdynlink_open" ;
    "caml_natdynlink_run" ;
    "caml_natdynlink_run_toplevel" ;
    "caml_natdynlink_loadsym"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Obj", [
    "caml_static_alloc" ;
    "caml_static_free" ;
    "caml_static_release_bytecode" ;
    "caml_static_resize" ;
    "caml_obj_is_block" ;
    "caml_obj_tag" ;
    "caml_obj_set_tag" ;
    "caml_obj_block" ;
    "caml_obj_dup" ;
    "caml_obj_truncate" ;
    "caml_obj_add_offset" ;
    "caml_lazy_follow_forward" ;
    "caml_lazy_make_forward" ;
    "caml_get_public_method"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Parsing", [
    "caml_parse_engine" ;
    "caml_set_parser_trace"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Signals", [
    "caml_install_signal_handler"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Stacks", [
    "caml_ensure_stack_capacity"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Str", [
    "caml_ml_string_length" ;
    "caml_create_string" ;
    "caml_string_get" ;
    "caml_string_set" ;
    "caml_string_equal" ;
    "caml_string_notequal" ;
    "caml_string_compare" ;
    "caml_string_lessthan" ;
    "caml_string_lessequal" ;
    "caml_string_greaterthan" ;
    "caml_string_greaterequal" ;
    "caml_blit_string" ;
    "caml_fill_string" ;
    "caml_is_printable" ;
    "caml_bitvect_test"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Sys", [
    "caml_sys_exit" ;
    "caml_sys_open" ;
    "caml_sys_close" ;
    "caml_sys_file_exists" ;
    "caml_sys_is_directory" ;
    "caml_sys_remove" ;
    "caml_sys_rename" ;
    "caml_sys_chdir" ;
    "caml_sys_getcwd" ;
    "caml_sys_getenv" ;
    "caml_sys_get_argv" ;
    "caml_sys_system_command" ;
    "caml_sys_time" ;
    "caml_sys_random_seed" ;
    "caml_sys_get_config" ;
    "caml_sys_read_directory"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Terminfo", [
    "caml_terminfo_setup" ;
    "caml_terminfo_backup" ;
    "caml_terminfo_standout" ;
    "caml_terminfo_resume"
  ]) ;
  ("fr.x9c.cadmium.primitives.stdlib.Weak", [
    "caml_weak_create" ;
    "caml_weak_set" ;
    "caml_weak_get" ;
    "caml_weak_get_copy" ;
    "caml_weak_check" ;
    "caml_weak_blit"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Applets", [
    "cadmium_applet_get_parameter"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Arrays", [
    "cadmium_array_new_instance" ;
    "cadmium_array_new_instance_dims" ;
    "cadmium_array_get_length" ;
    "cadmium_array_get" ;
    "cadmium_array_get_boolean" ;
    "cadmium_array_get_byte" ;
    "cadmium_array_get_char" ;
    "cadmium_array_get_double" ;
    "cadmium_array_get_float" ;
    "cadmium_array_get_int" ;
    "cadmium_array_get_long" ;
    "cadmium_array_get_short" ;
    "cadmium_array_set" ;
    "cadmium_array_set_boolean" ;
    "cadmium_array_set_byte" ;
    "cadmium_array_set_char" ;
    "cadmium_array_set_double" ;
    "cadmium_array_set_float" ;
    "cadmium_array_set_int" ;
    "cadmium_array_set_long" ;
    "cadmium_array_set_short"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Cadmium", [
    "cadmium_is_available" ;
    "cadmium_is_null" ;
    "cadmium_get_null" ;
    "cadmium_instance_of" ;
    "cadmium_object_of_string" ;
    "cadmium_java_char_of_char" ;
    "cadmium_char_of_java_char" ;
    "cadmium_equal" ;
    "cadmium_not_equal" ;
    "cadmium_print_stack_trace" ;
    "cadmium_throw_exception" ;
    "cadmium_synchronized" ;
    "cadmium_get_frame" ;
    "cadmium_get_canvas" ;
    "cadmium_get_buffer" ;
    "cadmium_get_binding" ;
    "cadmium_is_binding_object"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Classes", [
    "cadmium_class_cast" ;
    "cadmium_class_for_name" ;
    "cadmium_class_get_canonical_name" ;
    "cadmium_class_get_constructor" ;
    "cadmium_class_get_constructors" ;
    "cadmium_class_get_declared_constructor" ;
    "cadmium_class_get_declared_constructors" ;
    "cadmium_class_get_field" ;
    "cadmium_class_get_fields" ;
    "cadmium_class_get_declared_field" ;
    "cadmium_class_get_declared_fields" ;
    "cadmium_class_get_method" ;
    "cadmium_class_get_methods" ;
    "cadmium_class_get_declared_method" ;
    "cadmium_class_get_declared_methods" ;
    "cadmium_class_get_interfaces" ;
    "cadmium_class_get_modifiers" ;
    "cadmium_class_get_name" ;
    "cadmium_class_get_package" ;
    "cadmium_class_get_simple_name" ;
    "cadmium_class_get_superclass" ;
    "cadmium_class_is_array" ;
    "cadmium_class_is_interface" ;
    "cadmium_class_is_primitive" ;
    "cadmium_class_new_instance" ;
    "cadmium_class_as_class"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Constructors", [
    "cadmium_constr_get_declaring_class" ;
    "cadmium_constr_get_exception_types" ;
    "cadmium_constr_get_modifiers" ;
    "cadmium_constr_get_parameter_types" ;
    "cadmium_constr_new_instance" ;
    "cadmium_constr_as_constr"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.CreateInterface", [
    "cadmium_create_interface" ;
    "cadmium_create_named_interface"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Fields", [
    "cadmium_field_get_declaring_class" ;
    "cadmium_field_get_modifiers" ;
    "cadmium_field_get_name" ;
    "cadmium_field_get" ;
    "cadmium_field_get_boolean" ;
    "cadmium_field_get_byte" ;
    "cadmium_field_get_char" ;
    "cadmium_field_get_double" ;
    "cadmium_field_get_float" ;
    "cadmium_field_get_int" ;
    "cadmium_field_get_long" ;
    "cadmium_field_get_short" ;
    "cadmium_field_set" ;
    "cadmium_field_set_boolean" ;
    "cadmium_field_set_byte" ;
    "cadmium_field_set_char" ;
    "cadmium_field_set_double" ;
    "cadmium_field_set_float" ;
    "cadmium_field_set_int" ;
    "cadmium_field_set_long" ;
    "cadmium_field_set_short" ;
    "cadmium_field_as_field"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Methods", [
    "cadmium_meth_get_declaring_class" ;
    "cadmium_meth_get_exception_types" ;
    "cadmium_meth_get_modifiers" ;
    "cadmium_meth_get_name" ;
    "cadmium_meth_get_parameter_types" ;
    "cadmium_meth_get_return_type" ;
    "cadmium_meth_invoke" ;
    "cadmium_meth_as_meth"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Objects", [
    "cadmium_object_make" ;
    "cadmium_object_clone" ;
    "cadmium_object_equals" ;
    "cadmium_object_get_class" ;
    "cadmium_object_hash_code" ;
    "cadmium_object_notify" ;
    "cadmium_object_notify_all" ;
    "cadmium_object_to_string" ;
    "cadmium_object_wait" ;
    "cadmium_object_wait_time1" ;
    "cadmium_object_wait_time2"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Proxies", [
    "cadmium_register" ;
    "cadmium_register_object" ;
    "cadmium_register_object_transl"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmium.Runtime", [
    "cadmium_is_native" ;
    "cadmium_get_coderunner" ;
    "cadmium_get_context" ;
    "cadmium_get_parameters" ;
    "cadmium_make_interpreter" ;
    "cadmium_make_interpreter_native" ;
    "cadmium_interpreter_execute" ;
    "cadmium_interpreter_execute_callback" ;
    "cadmium_interpreter_get_context" ;
    "cadmium_interpreter_get_parameters"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Arrays", [
    "cadmiumjdbc_array_free" ;
    "cadmiumjdbc_array_get_array" ;
    "cadmiumjdbc_array_get_array_slice" ;
    "cadmiumjdbc_array_get_base_type" ;
    "cadmiumjdbc_array_get_base_type_name"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Blobs", [
    "cadmiumjdbc_blob_free" ;
    "cadmiumjdbc_blob_get_binary_stream" ;
    "cadmiumjdbc_blob_get_binary_stream_pos" ;
    "cadmiumjdbc_blob_get_bytes" ;
    "cadmiumjdbc_blob_length" ;
    "cadmiumjdbc_blob_position_blob" ;
    "cadmiumjdbc_blob_position_bytes" ;
    "cadmiumjdbc_blob_set_binary_stream" ;
    "cadmiumjdbc_blob_set_bytes" ;
    "cadmiumjdbc_blob_set_bytes_ofs" ;
    "cadmiumjdbc_blob_truncate"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.CallableStatements", [
    "cadmiumjdbc_callablestatement_get_array_idx" ;
    "cadmiumjdbc_callablestatement_get_array_name" ;
    "cadmiumjdbc_callablestatement_get_big_decimal_idx" ;
    "cadmiumjdbc_callablestatement_get_big_decimal_name" ;
    "cadmiumjdbc_callablestatement_get_blob_idx" ;
    "cadmiumjdbc_callablestatement_get_blob_name" ;
    "cadmiumjdbc_callablestatement_get_boolean_idx" ;
    "cadmiumjdbc_callablestatement_get_boolean_name" ;
    "cadmiumjdbc_callablestatement_get_byte_idx" ;
    "cadmiumjdbc_callablestatement_get_byte_name" ;
    "cadmiumjdbc_callablestatement_get_bytes_idx" ;
    "cadmiumjdbc_callablestatement_get_bytes_name" ;
    "cadmiumjdbc_callablestatement_get_clob_idx" ;
    "cadmiumjdbc_callablestatement_get_clob_name" ;
    "cadmiumjdbc_callablestatement_get_date_idx" ;
    "cadmiumjdbc_callablestatement_get_date_name" ;
    "cadmiumjdbc_callablestatement_get_double_idx" ;
    "cadmiumjdbc_callablestatement_get_double_name" ;
    "cadmiumjdbc_callablestatement_get_float_idx" ;
    "cadmiumjdbc_callablestatement_get_float_name" ;
    "cadmiumjdbc_callablestatement_get_int_idx" ;
    "cadmiumjdbc_callablestatement_get_int_name" ;
    "cadmiumjdbc_callablestatement_get_long_idx" ;
    "cadmiumjdbc_callablestatement_get_long_name" ;
    "cadmiumjdbc_callablestatement_get_nclob_idx" ;
    "cadmiumjdbc_callablestatement_get_nclob_name" ;
    "cadmiumjdbc_callablestatement_get_nstring_idx" ;
    "cadmiumjdbc_callablestatement_get_nstring_name" ;
    "cadmiumjdbc_callablestatement_get_object_idx" ;
    "cadmiumjdbc_callablestatement_get_object_name" ;
    "cadmiumjdbc_callablestatement_get_ref_idx" ;
    "cadmiumjdbc_callablestatement_get_ref_name" ;
    "cadmiumjdbc_callablestatement_get_row_id_idx" ;
    "cadmiumjdbc_callablestatement_get_row_id_name" ;
    "cadmiumjdbc_callablestatement_get_short_idx" ;
    "cadmiumjdbc_callablestatement_get_short_name" ;
    "cadmiumjdbc_callablestatement_get_sqlxml_idx" ;
    "cadmiumjdbc_callablestatement_get_sqlxml_name" ;
    "cadmiumjdbc_callablestatement_get_string_idx" ;
    "cadmiumjdbc_callablestatement_get_string_name" ;
    "cadmiumjdbc_callablestatement_get_time_idx" ;
    "cadmiumjdbc_callablestatement_get_time_name" ;
    "cadmiumjdbc_callablestatement_get_timestamp_idx" ;
    "cadmiumjdbc_callablestatement_get_timestamp_name" ;
    "cadmiumjdbc_callablestatement_get_url_idx" ;
    "cadmiumjdbc_callablestatement_get_url_name" ;
    "cadmiumjdbc_callablestatement_register_out_parameter_idx" ;
    "cadmiumjdbc_callablestatement_register_out_parameter_scale_idx" ;
    "cadmiumjdbc_callablestatement_register_out_parameter_name" ;
    "cadmiumjdbc_callablestatement_register_out_parameter_scale_name" ;
    "cadmiumjdbc_callablestatement_set_ascii_stream_name" ;
    "cadmiumjdbc_callablestatement_set_ascii_stream_length_name" ;
    "cadmiumjdbc_callablestatement_set_ascii_stream_length64_name" ;
    "cadmiumjdbc_callablestatement_set_big_decimal_name" ;
    "cadmiumjdbc_callablestatement_set_binary_stream_name" ;
    "cadmiumjdbc_callablestatement_set_binary_stream_length_name" ;
    "cadmiumjdbc_callablestatement_set_binary_stream_length64_name" ;
    "cadmiumjdbc_callablestatement_set_blob_name" ;
    "cadmiumjdbc_callablestatement_set_boolean_name" ;
    "cadmiumjdbc_callablestatement_set_byte_name" ;
    "cadmiumjdbc_callablestatement_set_bytes_name" ;
    "cadmiumjdbc_callablestatement_set_clob_name" ;
    "cadmiumjdbc_callablestatement_set_date_name" ;
    "cadmiumjdbc_callablestatement_set_double_name" ;
    "cadmiumjdbc_callablestatement_set_float_name" ;
    "cadmiumjdbc_callablestatement_set_int_name" ;
    "cadmiumjdbc_callablestatement_set_long_name" ;
    "cadmiumjdbc_callablestatement_set_nclob_name" ;
    "cadmiumjdbc_callablestatement_set_nstring_name" ;
    "cadmiumjdbc_callablestatement_set_object_name" ;
    "cadmiumjdbc_callablestatement_set_object_type_name" ;
    "cadmiumjdbc_callablestatement_set_object_type_scale_name" ;
    "cadmiumjdbc_callablestatement_set_row_id_name" ;
    "cadmiumjdbc_callablestatement_set_short_name" ;
    "cadmiumjdbc_callablestatement_set_sqlxml_name" ;
    "cadmiumjdbc_callablestatement_set_string_name" ;
    "cadmiumjdbc_callablestatement_set_time_name" ;
    "cadmiumjdbc_callablestatement_set_timestamp_name" ;
    "cadmiumjdbc_callablestatement_set_url_name"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Clobs", [
    "cadmiumjdbc_clob_free" ;
    "cadmiumjdbc_clob_get_ascii_stream" ;
    "cadmiumjdbc_clob_get_sub_string" ;
    "cadmiumjdbc_clob_length" ;
    "cadmiumjdbc_clob_position_clob" ;
    "cadmiumjdbc_clob_position_string" ;
    "cadmiumjdbc_clob_set_ascii_stream" ;
    "cadmiumjdbc_clob_set_string" ;
    "cadmiumjdbc_clob_set_string_ofs" ;
    "cadmiumjdbc_clob_truncate"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Connections", [
    "cadmiumjdbc_connection_clear_warnings" ;
    "cadmiumjdbc_connection_close" ;
    "cadmiumjdbc_connection_commit" ;
    "cadmiumjdbc_connection_create_array_of" ;
    "cadmiumjdbc_connection_create_blob" ;
    "cadmiumjdbc_connection_create_clob" ;
    "cadmiumjdbc_connection_create_nclob" ;
    "cadmiumjdbc_connection_create_sqlxml" ;
    "cadmiumjdbc_connection_create_statement" ;
    "cadmiumjdbc_connection_create_statement_type_conc" ;
    "cadmiumjdbc_connection_create_statement_type_conc_hold" ;
    "cadmiumjdbc_connection_create_struct" ;
    "cadmiumjdbc_connection_get_auto_commit" ;
    "cadmiumjdbc_connection_get_catalog" ;
    "cadmiumjdbc_connection_get_client_infos" ;
    "cadmiumjdbc_connection_get_client_info" ;
    "cadmiumjdbc_connection_get_holdability" ;
    "cadmiumjdbc_connection_get_meta_data" ;
    "cadmiumjdbc_connection_get_transaction_isolation" ;
    "cadmiumjdbc_connection_get_warnings" ;
    "cadmiumjdbc_connection_is_closed" ;
    "cadmiumjdbc_connection_is_read_only" ;
    "cadmiumjdbc_connection_is_valid" ;
    "cadmiumjdbc_connection_native_sql" ;
    "cadmiumjdbc_connection_prepare_call" ;
    "cadmiumjdbc_connection_prepare_call_type_conc" ;
    "cadmiumjdbc_connection_prepare_call_type_conc_hold" ;
    "cadmiumjdbc_connection_prepare_statement" ;
    "cadmiumjdbc_connection_prepare_statement_autogen_keys" ;
    "cadmiumjdbc_connection_prepare_statement_indexes" ;
    "cadmiumjdbc_connection_prepare_statement_type_conc" ;
    "cadmiumjdbc_connection_prepare_statement_type_conc_hold" ;
    "cadmiumjdbc_connection_prepare_statement_names" ;
    "cadmiumjdbc_connection_release_savepoint" ;
    "cadmiumjdbc_connection_rollback" ;
    "cadmiumjdbc_connection_rollback_to" ;
    "cadmiumjdbc_connection_set_auto_commit" ;
    "cadmiumjdbc_connection_set_catalog" ;
    "cadmiumjdbc_connection_set_client_infos" ;
    "cadmiumjdbc_connection_set_client_info" ;
    "cadmiumjdbc_connection_set_holdability" ;
    "cadmiumjdbc_connection_set_read_only" ;
    "cadmiumjdbc_connection_set_savepoint" ;
    "cadmiumjdbc_connection_set_savepoint_with_name" ;
    "cadmiumjdbc_connection_set_transaction_isolation"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.DatabaseMetaDatas", [
    "cadmiumjdbc_databasemetadata_all_procedures_are_callable" ;
    "cadmiumjdbc_databasemetadata_all_tables_are_selectable" ;
    "cadmiumjdbc_databasemetadata_auto_commit_failure_closes_all_result_sets" ;
    "cadmiumjdbc_databasemetadata_data_definition_causes_transaction_commit" ;
    "cadmiumjdbc_databasemetadata_data_definition_ignored_in_transactions" ;
    "cadmiumjdbc_databasemetadata_deletes_are_detected" ;
    "cadmiumjdbc_databasemetadata_does_max_row_size_include_blobs" ;
    "cadmiumjdbc_databasemetadata_get_attributes" ;
    "cadmiumjdbc_databasemetadata_get_best_row_identifier" ;
    "cadmiumjdbc_databasemetadata_get_best_row_identifier_native" ;
    "cadmiumjdbc_databasemetadata_get_catalogs" ;
    "cadmiumjdbc_databasemetadata_get_catalog_separator" ;
    "cadmiumjdbc_databasemetadata_get_catalog_term" ;
    "cadmiumjdbc_databasemetadata_get_client_info_properties" ;
    "cadmiumjdbc_databasemetadata_get_column_privileges" ;
    "cadmiumjdbc_databasemetadata_get_columns" ;
    "cadmiumjdbc_databasemetadata_get_cross_reference" ;
    "cadmiumjdbc_databasemetadata_get_cross_reference_native" ;
    "cadmiumjdbc_databasemetadata_get_database_major_version" ;
    "cadmiumjdbc_databasemetadata_get_database_minor_version" ;
    "cadmiumjdbc_databasemetadata_get_database_product_name" ;
    "cadmiumjdbc_databasemetadata_get_database_product_version" ;
    "cadmiumjdbc_databasemetadata_get_default_transaction_isolation" ;
    "cadmiumjdbc_databasemetadata_get_driver_major_version" ;
    "cadmiumjdbc_databasemetadata_get_driver_minor_version" ;
    "cadmiumjdbc_databasemetadata_get_driver_name" ;
    "cadmiumjdbc_databasemetadata_get_driver_version" ;
    "cadmiumjdbc_databasemetadata_get_exported_keys" ;
    "cadmiumjdbc_databasemetadata_get_extra_name_characters" ;
    "cadmiumjdbc_databasemetadata_get_function_columns" ;
    "cadmiumjdbc_databasemetadata_get_functions" ;
    "cadmiumjdbc_databasemetadata_get_identifier_quote_string" ;
    "cadmiumjdbc_databasemetadata_get_imported_keys" ;
    "cadmiumjdbc_databasemetadata_get_index_info" ;
    "cadmiumjdbc_databasemetadata_get_index_info_native" ;
    "cadmiumjdbc_databasemetadata_get_jdbc_major_version" ;
    "cadmiumjdbc_databasemetadata_get_jdbc_minor_version" ;
    "cadmiumjdbc_databasemetadata_get_max_binary_literal_length" ;
    "cadmiumjdbc_databasemetadata_get_max_catalog_name_length" ;
    "cadmiumjdbc_databasemetadata_get_max_char_literal_length" ;
    "cadmiumjdbc_databasemetadata_get_max_column_name_length" ;
    "cadmiumjdbc_databasemetadata_get_max_columns_in_group_by" ;
    "cadmiumjdbc_databasemetadata_get_max_columns_in_index" ;
    "cadmiumjdbc_databasemetadata_get_max_columns_in_order_by" ;
    "cadmiumjdbc_databasemetadata_get_max_columns_in_select" ;
    "cadmiumjdbc_databasemetadata_get_max_columns_in_table" ;
    "cadmiumjdbc_databasemetadata_get_max_connections" ;
    "cadmiumjdbc_databasemetadata_get_max_cursor_name_length" ;
    "cadmiumjdbc_databasemetadata_get_max_index_length" ;
    "cadmiumjdbc_databasemetadata_get_max_procedure_name_length" ;
    "cadmiumjdbc_databasemetadata_get_max_row_size" ;
    "cadmiumjdbc_databasemetadata_get_max_schema_name_length" ;
    "cadmiumjdbc_databasemetadata_get_max_statement_length" ;
    "cadmiumjdbc_databasemetadata_get_max_statements" ;
    "cadmiumjdbc_databasemetadata_get_max_table_name_length" ;
    "cadmiumjdbc_databasemetadata_get_max_tables_in_select" ;
    "cadmiumjdbc_databasemetadata_get_max_user_name_length" ;
    "cadmiumjdbc_databasemetadata_get_numeric_functions" ;
    "cadmiumjdbc_databasemetadata_get_primary_keys" ;
    "cadmiumjdbc_databasemetadata_get_procedure_columns" ;
    "cadmiumjdbc_databasemetadata_get_procedures" ;
    "cadmiumjdbc_databasemetadata_get_procedure_term" ;
    "cadmiumjdbc_databasemetadata_get_result_set_holdability" ;
    "cadmiumjdbc_databasemetadata_get_row_id_lifetime" ;
    "cadmiumjdbc_databasemetadata_get_schemas" ;
    "cadmiumjdbc_databasemetadata_get_schemas_params" ;
    "cadmiumjdbc_databasemetadata_get_schema_term" ;
    "cadmiumjdbc_databasemetadata_get_search_string_escape" ;
    "cadmiumjdbc_databasemetadata_get_sql_keywords" ;
    "cadmiumjdbc_databasemetadata_get_sql_state_type" ;
    "cadmiumjdbc_databasemetadata_get_string_functions" ;
    "cadmiumjdbc_databasemetadata_get_super_tables" ;
    "cadmiumjdbc_databasemetadata_get_super_types" ;
    "cadmiumjdbc_databasemetadata_get_system_functions" ;
    "cadmiumjdbc_databasemetadata_get_table_privileges" ;
    "cadmiumjdbc_databasemetadata_get_tables" ;
    "cadmiumjdbc_databasemetadata_get_table_types" ;
    "cadmiumjdbc_databasemetadata_get_time_date_functions" ;
    "cadmiumjdbc_databasemetadata_get_type_info" ;
    "cadmiumjdbc_databasemetadata_get_udts" ;
    "cadmiumjdbc_databasemetadata_get_url" ;
    "cadmiumjdbc_databasemetadata_get_user_name" ;
    "cadmiumjdbc_databasemetadata_get_version_columns" ;
    "cadmiumjdbc_databasemetadata_inserts_are_detected" ;
    "cadmiumjdbc_databasemetadata_is_catalog_at_start" ;
    "cadmiumjdbc_databasemetadata_is_read_only" ;
    "cadmiumjdbc_databasemetadata_locators_update_copy" ;
    "cadmiumjdbc_databasemetadata_null_plus_non_null_is_null" ;
    "cadmiumjdbc_databasemetadata_nulls_are_sorted_at_end" ;
    "cadmiumjdbc_databasemetadata_nulls_are_sorted_at_start" ;
    "cadmiumjdbc_databasemetadata_nulls_are_sorted_high" ;
    "cadmiumjdbc_databasemetadata_nulls_are_sorted_low" ;
    "cadmiumjdbc_databasemetadata_others_deletes_are_visible" ;
    "cadmiumjdbc_databasemetadata_others_inserts_are_visible" ;
    "cadmiumjdbc_databasemetadata_others_updates_are_visible" ;
    "cadmiumjdbc_databasemetadata_own_deletes_are_visible" ;
    "cadmiumjdbc_databasemetadata_own_inserts_are_visible" ;
    "cadmiumjdbc_databasemetadata_own_updates_are_visible" ;
    "cadmiumjdbc_databasemetadata_stores_lower_case_identifiers" ;
    "cadmiumjdbc_databasemetadata_stores_lower_case_quoted_identifiers" ;
    "cadmiumjdbc_databasemetadata_stores_mixed_case_identifiers" ;
    "cadmiumjdbc_databasemetadata_stores_mixed_case_quoted_identifiers" ;
    "cadmiumjdbc_databasemetadata_stores_upper_case_identifiers" ;
    "cadmiumjdbc_databasemetadata_stores_upper_case_quoted_identifiers" ;
    "cadmiumjdbc_databasemetadata_supports_alter_table_with_add_column" ;
    "cadmiumjdbc_databasemetadata_supports_alter_table_with_drop_column" ;
    "cadmiumjdbc_databasemetadata_supports_ansi92_entry_level_sql" ;
    "cadmiumjdbc_databasemetadata_supports_ansi92_full_sql" ;
    "cadmiumjdbc_databasemetadata_supports_ansi92_intermediate_sql" ;
    "cadmiumjdbc_databasemetadata_supports_batch_updates" ;
    "cadmiumjdbc_databasemetadata_supports_catalogs_in_data_manipulation" ;
    "cadmiumjdbc_databasemetadata_supports_catalogs_in_index_definitions" ;
    "cadmiumjdbc_databasemetadata_supports_catalogs_in_privilege_definitions" ;
    "cadmiumjdbc_databasemetadata_supports_catalogs_in_procedure_calls" ;
    "cadmiumjdbc_databasemetadata_supports_catalogs_in_table_definitions" ;
    "cadmiumjdbc_databasemetadata_supports_column_aliasing" ;
    "cadmiumjdbc_databasemetadata_supports_convert" ;
    "cadmiumjdbc_databasemetadata_supports_convert_types" ;
    "cadmiumjdbc_databasemetadata_supports_core_sql_grammar" ;
    "cadmiumjdbc_databasemetadata_supports_correlated_subqueries" ;
    "cadmiumjdbc_databasemetadata_supports_data_definition_and_data_manipulation_transactions" ;
    "cadmiumjdbc_databasemetadata_supports_data_manipulation_transactions_only" ;
    "cadmiumjdbc_databasemetadata_supports_different_table_correlation_names" ;
    "cadmiumjdbc_databasemetadata_supports_expressions_in_order_by" ;
    "cadmiumjdbc_databasemetadata_supports_extended_sql_grammar" ;
    "cadmiumjdbc_databasemetadata_supports_full_outer_joins" ;
    "cadmiumjdbc_databasemetadata_supports_get_generated_keys" ;
    "cadmiumjdbc_databasemetadata_supports_group_by" ;
    "cadmiumjdbc_databasemetadata_supports_group_by_beyond_select" ;
    "cadmiumjdbc_databasemetadata_supports_group_by_unrelated" ;
    "cadmiumjdbc_databasemetadata_supports_integrity_enhancement_facility" ;
    "cadmiumjdbc_databasemetadata_supports_like_escape_clause" ;
    "cadmiumjdbc_databasemetadata_supports_limited_outer_joins" ;
    "cadmiumjdbc_databasemetadata_supports_minimum_sql_grammar" ;
    "cadmiumjdbc_databasemetadata_supports_mixed_case_identifiers" ;
    "cadmiumjdbc_databasemetadata_supports_mixed_case_quoted_identifiers" ;
    "cadmiumjdbc_databasemetadata_supports_multiple_open_results" ;
    "cadmiumjdbc_databasemetadata_supports_multiple_result_sets" ;
    "cadmiumjdbc_databasemetadata_supports_multiple_transactions" ;
    "cadmiumjdbc_databasemetadata_supports_named_parameters" ;
    "cadmiumjdbc_databasemetadata_supports_non_nullable_columns" ;
    "cadmiumjdbc_databasemetadata_supports_open_cursors_across_commit" ;
    "cadmiumjdbc_databasemetadata_supports_open_cursors_across_rollback" ;
    "cadmiumjdbc_databasemetadata_supports_open_statements_across_commit" ;
    "cadmiumjdbc_databasemetadata_supports_open_statements_across_rollback" ;
    "cadmiumjdbc_databasemetadata_supports_order_by_unrelated" ;
    "cadmiumjdbc_databasemetadata_supports_outer_joins" ;
    "cadmiumjdbc_databasemetadata_supports_positioned_delete" ;
    "cadmiumjdbc_databasemetadata_supports_positioned_update" ;
    "cadmiumjdbc_databasemetadata_supports_result_set_concurrency" ;
    "cadmiumjdbc_databasemetadata_supports_result_set_holdability" ;
    "cadmiumjdbc_databasemetadata_supports_result_set_type" ;
    "cadmiumjdbc_databasemetadata_supports_savepoints" ;
    "cadmiumjdbc_databasemetadata_supports_schemas_in_data_manipulation" ;
    "cadmiumjdbc_databasemetadata_supports_schemas_in_index_definitions" ;
    "cadmiumjdbc_databasemetadata_supports_schemas_in_privilege_definitions" ;
    "cadmiumjdbc_databasemetadata_supports_schemas_in_procedure_calls" ;
    "cadmiumjdbc_databasemetadata_supports_schemas_in_table_definitions" ;
    "cadmiumjdbc_databasemetadata_supports_select_for_update" ;
    "cadmiumjdbc_databasemetadata_supports_statement_pooling" ;
    "cadmiumjdbc_databasemetadata_supports_stored_functions_using_call_syntax" ;
    "cadmiumjdbc_databasemetadata_supports_stored_procedures" ;
    "cadmiumjdbc_databasemetadata_supports_subqueries_in_comparisons" ;
    "cadmiumjdbc_databasemetadata_supports_subqueries_in_exists" ;
    "cadmiumjdbc_databasemetadata_supports_subqueries_in_ins" ;
    "cadmiumjdbc_databasemetadata_supports_subqueries_in_quantifieds" ;
    "cadmiumjdbc_databasemetadata_supports_table_correlation_names" ;
    "cadmiumjdbc_databasemetadata_supports_transaction_isolation_level" ;
    "cadmiumjdbc_databasemetadata_supports_transactions" ;
    "cadmiumjdbc_databasemetadata_supports_union" ;
    "cadmiumjdbc_databasemetadata_supports_union_all" ;
    "cadmiumjdbc_databasemetadata_updates_are_detected" ;
    "cadmiumjdbc_databasemetadata_uses_local_file_per_table" ;
    "cadmiumjdbc_databasemetadata_uses_local_files"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.DateBase", [
    "cadmiumjdbc_datebase_after" ;
    "cadmiumjdbc_datebase_before" ;
    "cadmiumjdbc_datebase_compare_to" ;
    "cadmiumjdbc_datebase_equals" ;
    "cadmiumjdbc_datebase_get_day_of_month" ;
    "cadmiumjdbc_datebase_get_day_of_week" ;
    "cadmiumjdbc_datebase_get_day_of_week_in_month" ;
    "cadmiumjdbc_datebase_get_day_of_year" ;
    "cadmiumjdbc_datebase_get_hour_of_day" ;
    "cadmiumjdbc_datebase_get_millisecond" ;
    "cadmiumjdbc_datebase_get_minute" ;
    "cadmiumjdbc_datebase_get_month" ;
    "cadmiumjdbc_datebase_get_second" ;
    "cadmiumjdbc_datebase_get_week_of_month" ;
    "cadmiumjdbc_datebase_get_week_of_year" ;
    "cadmiumjdbc_datebase_get_year" ;
    "cadmiumjdbc_datebase_set_day_of_month" ;
    "cadmiumjdbc_datebase_set_day_of_week" ;
    "cadmiumjdbc_datebase_set_day_of_week_in_month" ;
    "cadmiumjdbc_datebase_set_day_of_year" ;
    "cadmiumjdbc_datebase_set_hour_of_day" ;
    "cadmiumjdbc_datebase_set_millisecond" ;
    "cadmiumjdbc_datebase_set_minute" ;
    "cadmiumjdbc_datebase_set_month" ;
    "cadmiumjdbc_datebase_set_second" ;
    "cadmiumjdbc_datebase_set_week_of_month" ;
    "cadmiumjdbc_datebase_set_week_of_year" ;
    "cadmiumjdbc_datebase_set_year"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Dates", [
    "cadmiumjdbc_date_make" ;
    "cadmiumjdbc_date_make_of_int64" ;
    "cadmiumjdbc_date_make_of_string" ;
    "cadmiumjdbc_date_get_time" ;
    "cadmiumjdbc_date_set_time" ;
    "cadmiumjdbc_date_to_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.DriverManagers", [
    "cadmiumjdbc_drivermanager_deregister_driver" ;
    "cadmiumjdbc_drivermanager_get_connection" ;
    "cadmiumjdbc_drivermanager_get_connection_with_properties" ;
    "cadmiumjdbc_drivermanager_get_connection_with_user_passwd" ;
    "cadmiumjdbc_drivermanager_get_driver" ;
    "cadmiumjdbc_drivermanager_get_drivers" ;
    "cadmiumjdbc_drivermanager_get_login_timeout" ;
    "cadmiumjdbc_drivermanager_print_log" ;
    "cadmiumjdbc_drivermanager_register_driver" ;
    "cadmiumjdbc_drivermanager_set_login_timeout"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Drivers", [
    "cadmiumjdbc_driver_accepts_url" ;
    "cadmiumjdbc_driver_connect" ;
    "cadmiumjdbc_driver_get_major_version" ;
    "cadmiumjdbc_driver_get_minor_version" ;
    "cadmiumjdbc_driver_get_property_info" ;
    "cadmiumjdbc_driver_is_jdbc_compliant"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.ParameterMetaDatas", [
    "cadmiumjdbc_parametermetadata_get_parameter_class_name" ;
    "cadmiumjdbc_parametermetadata_get_parameter_count" ;
    "cadmiumjdbc_parametermetadata_get_parameter_mode" ;
    "cadmiumjdbc_parametermetadata_get_parameter_type" ;
    "cadmiumjdbc_parametermetadata_get_parameter_type_name" ;
    "cadmiumjdbc_parametermetadata_get_precision" ;
    "cadmiumjdbc_parametermetadata_get_scale" ;
    "cadmiumjdbc_parametermetadata_is_nullable" ;
    "cadmiumjdbc_parametermetadata_is_signed"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.PreparedStatements", [
    "cadmiumjdbc_preparedstatement_add_batch" ;
    "cadmiumjdbc_preparedstatement_clear_parameters" ;
    "cadmiumjdbc_preparedstatement_execute" ;
    "cadmiumjdbc_preparedstatement_execute_query" ;
    "cadmiumjdbc_preparedstatement_execute_update" ;
    "cadmiumjdbc_preparedstatement_get_meta_data" ;
    "cadmiumjdbc_preparedstatement_get_parameter_meta_data" ;
    "cadmiumjdbc_preparedstatement_set_array_idx" ;
    "cadmiumjdbc_preparedstatement_set_ascii_stream_idx" ;
    "cadmiumjdbc_preparedstatement_set_ascii_stream_length_idx" ;
    "cadmiumjdbc_preparedstatement_set_ascii_stream_length64_idx" ;
    "cadmiumjdbc_preparedstatement_set_big_decimal_idx" ;
    "cadmiumjdbc_preparedstatement_set_binary_stream_idx" ;
    "cadmiumjdbc_preparedstatement_set_binary_stream_length_idx" ;
    "cadmiumjdbc_preparedstatement_set_binary_stream_length64_idx" ;
    "cadmiumjdbc_preparedstatement_set_blob_idx" ;
    "cadmiumjdbc_preparedstatement_set_boolean_idx" ;
    "cadmiumjdbc_preparedstatement_set_byte_idx" ;
    "cadmiumjdbc_preparedstatement_set_bytes_idx" ;
    "cadmiumjdbc_preparedstatement_set_clob_idx" ;
    "cadmiumjdbc_preparedstatement_set_date_idx" ;
    "cadmiumjdbc_preparedstatement_set_double_idx" ;
    "cadmiumjdbc_preparedstatement_set_float_idx" ;
    "cadmiumjdbc_preparedstatement_set_int_idx" ;
    "cadmiumjdbc_preparedstatement_set_long_idx" ;
    "cadmiumjdbc_preparedstatement_set_nclob_idx" ;
    "cadmiumjdbc_preparedstatement_set_nstring_idx" ;
    "cadmiumjdbc_preparedstatement_set_object_idx" ;
    "cadmiumjdbc_preparedstatement_set_object_type_idx" ;
    "cadmiumjdbc_preparedstatement_set_object_type_scale_idx" ;
    "cadmiumjdbc_preparedstatement_set_ref_idx" ;
    "cadmiumjdbc_preparedstatement_set_row_id_idx" ;
    "cadmiumjdbc_preparedstatement_set_short_idx" ;
    "cadmiumjdbc_preparedstatement_set_sqlxml_idx" ;
    "cadmiumjdbc_preparedstatement_set_string_idx" ;
    "cadmiumjdbc_preparedstatement_set_time_idx" ;
    "cadmiumjdbc_preparedstatement_set_timestamp_idx" ;
    "cadmiumjdbc_preparedstatement_set_url_idx"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Refs", [
    "cadmiumjdbc_ref_get_base_type_name" ;
    "cadmiumjdbc_ref_get_object" ;
    "cadmiumjdbc_ref_set_object"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.ResultSetMetaDatas", [
    "cadmiumjdbc_resultsetmetadata_get_catalog_name" ;
    "cadmiumjdbc_resultsetmetadata_get_column_class_name" ;
    "cadmiumjdbc_resultsetmetadata_get_column_count" ;
    "cadmiumjdbc_resultsetmetadata_get_column_display_size" ;
    "cadmiumjdbc_resultsetmetadata_get_column_label" ;
    "cadmiumjdbc_resultsetmetadata_get_column_name" ;
    "cadmiumjdbc_resultsetmetadata_get_column_type" ;
    "cadmiumjdbc_resultsetmetadata_get_column_type_name" ;
    "cadmiumjdbc_resultsetmetadata_get_precision" ;
    "cadmiumjdbc_resultsetmetadata_get_scale" ;
    "cadmiumjdbc_resultsetmetadata_get_schema_name" ;
    "cadmiumjdbc_resultsetmetadata_get_table_name" ;
    "cadmiumjdbc_resultsetmetadata_is_auto_increment" ;
    "cadmiumjdbc_resultsetmetadata_is_case_sensitive" ;
    "cadmiumjdbc_resultsetmetadata_is_currency" ;
    "cadmiumjdbc_resultsetmetadata_is_definitely_writable" ;
    "cadmiumjdbc_resultsetmetadata_is_nullable" ;
    "cadmiumjdbc_resultsetmetadata_is_read_only" ;
    "cadmiumjdbc_resultsetmetadata_is_searchable" ;
    "cadmiumjdbc_resultsetmetadata_is_signed" ;
    "cadmiumjdbc_resultsetmetadata_is_writable"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.ResultSets", [
    "cadmiumjdbc_resultset_absolute" ;
    "cadmiumjdbc_resultset_after_last" ;
    "cadmiumjdbc_resultset_before_first" ;
    "cadmiumjdbc_resultset_cancel_row_updates" ;
    "cadmiumjdbc_resultset_clear_warnings" ;
    "cadmiumjdbc_resultset_close" ;
    "cadmiumjdbc_resultset_delete_row" ;
    "cadmiumjdbc_resultset_find_column" ;
    "cadmiumjdbc_resultset_first" ;
    "cadmiumjdbc_resultset_get_array_idx" ;
    "cadmiumjdbc_resultset_get_array_name" ;
    "cadmiumjdbc_resultset_get_ascii_stream_idx" ;
    "cadmiumjdbc_resultset_get_ascii_stream_name" ;
    "cadmiumjdbc_resultset_get_big_decimal_idx" ;
    "cadmiumjdbc_resultset_get_big_decimal_name" ;
    "cadmiumjdbc_resultset_get_binary_stream_idx" ;
    "cadmiumjdbc_resultset_get_binary_stream_name" ;
    "cadmiumjdbc_resultset_get_blob_idx" ;
    "cadmiumjdbc_resultset_get_blob_name" ;
    "cadmiumjdbc_resultset_get_boolean_idx" ;
    "cadmiumjdbc_resultset_get_boolean_name" ;
    "cadmiumjdbc_resultset_get_byte_idx" ;
    "cadmiumjdbc_resultset_get_byte_name" ;
    "cadmiumjdbc_resultset_get_bytes_idx" ;
    "cadmiumjdbc_resultset_get_bytes_name" ;
    "cadmiumjdbc_resultset_get_clob_idx" ;
    "cadmiumjdbc_resultset_get_clob_name" ;
    "cadmiumjdbc_resultset_get_concurrency" ;
    "cadmiumjdbc_resultset_get_cursor_name" ;
    "cadmiumjdbc_resultset_get_date_idx" ;
    "cadmiumjdbc_resultset_get_date_name" ;
    "cadmiumjdbc_resultset_get_double_idx" ;
    "cadmiumjdbc_resultset_get_double_name" ;
    "cadmiumjdbc_resultset_get_fetch_direction" ;
    "cadmiumjdbc_resultset_get_fetch_size" ;
    "cadmiumjdbc_resultset_get_float_idx" ;
    "cadmiumjdbc_resultset_get_float_name" ;
    "cadmiumjdbc_resultset_get_holdability" ;
    "cadmiumjdbc_resultset_get_int_idx" ;
    "cadmiumjdbc_resultset_get_int_name" ;
    "cadmiumjdbc_resultset_get_long_idx" ;
    "cadmiumjdbc_resultset_get_long_name" ;
    "cadmiumjdbc_resultset_get_meta_data" ;
    "cadmiumjdbc_resultset_get_nclob_idx" ;
    "cadmiumjdbc_resultset_get_nclob_name" ;
    "cadmiumjdbc_resultset_get_nstring_idx" ;
    "cadmiumjdbc_resultset_get_nstring_name" ;
    "cadmiumjdbc_resultset_get_object_idx" ;
    "cadmiumjdbc_resultset_get_object_name" ;
    "cadmiumjdbc_resultset_get_ref_idx" ;
    "cadmiumjdbc_resultset_get_ref_name" ;
    "cadmiumjdbc_resultset_get_row" ;
    "cadmiumjdbc_resultset_get_row_id_idx" ;
    "cadmiumjdbc_resultset_get_row_id_name" ;
    "cadmiumjdbc_resultset_get_short_idx" ;
    "cadmiumjdbc_resultset_get_short_name" ;
    "cadmiumjdbc_resultset_get_sqlxml_idx" ;
    "cadmiumjdbc_resultset_get_sqlxml_name" ;
    "cadmiumjdbc_resultset_get_string_idx" ;
    "cadmiumjdbc_resultset_get_string_name" ;
    "cadmiumjdbc_resultset_get_time_idx" ;
    "cadmiumjdbc_resultset_get_time_name" ;
    "cadmiumjdbc_resultset_get_timestamp_idx" ;
    "cadmiumjdbc_resultset_get_timestamp_name" ;
    "cadmiumjdbc_resultset_get_type" ;
    "cadmiumjdbc_resultset_get_url_idx" ;
    "cadmiumjdbc_resultset_get_url_name" ;
    "cadmiumjdbc_resultset_get_warnings" ;
    "cadmiumjdbc_resultset_insert_row" ;
    "cadmiumjdbc_resultset_is_after_last" ;
    "cadmiumjdbc_resultset_is_before_first" ;
    "cadmiumjdbc_resultset_is_closed" ;
    "cadmiumjdbc_resultset_is_first" ;
    "cadmiumjdbc_resultset_is_last" ;
    "cadmiumjdbc_resultset_last" ;
    "cadmiumjdbc_resultset_move_to_current_row" ;
    "cadmiumjdbc_resultset_move_to_insert_row" ;
    "cadmiumjdbc_resultset_next" ;
    "cadmiumjdbc_resultset_previous" ;
    "cadmiumjdbc_resultset_refresh_row" ;
    "cadmiumjdbc_resultset_relative" ;
    "cadmiumjdbc_resultset_row_deleted" ;
    "cadmiumjdbc_resultset_row_inserted" ;
    "cadmiumjdbc_resultset_row_updated" ;
    "cadmiumjdbc_resultset_set_fetch_direction" ;
    "cadmiumjdbc_resultset_set_fetch_size" ;
    "cadmiumjdbc_resultset_update_array_idx" ;
    "cadmiumjdbc_resultset_update_array_name" ;
    "cadmiumjdbc_resultset_update_ascii_stream_idx" ;
    "cadmiumjdbc_resultset_update_ascii_stream_name" ;
    "cadmiumjdbc_resultset_update_ascii_stream_length_idx" ;
    "cadmiumjdbc_resultset_update_ascii_stream_length_name" ;
    "cadmiumjdbc_resultset_update_ascii_stream_length64_idx" ;
    "cadmiumjdbc_resultset_update_ascii_stream_length64_name" ;
    "cadmiumjdbc_resultset_update_big_decimal_idx" ;
    "cadmiumjdbc_resultset_update_big_decimal_name" ;
    "cadmiumjdbc_resultset_update_binary_stream_idx" ;
    "cadmiumjdbc_resultset_update_binary_stream_name" ;
    "cadmiumjdbc_resultset_update_binary_stream_length_idx" ;
    "cadmiumjdbc_resultset_update_binary_stream_length_name" ;
    "cadmiumjdbc_resultset_update_binary_stream_length64_idx" ;
    "cadmiumjdbc_resultset_update_binary_stream_length64_name" ;
    "cadmiumjdbc_resultset_update_blob_idx" ;
    "cadmiumjdbc_resultset_update_blob_name" ;
    "cadmiumjdbc_resultset_update_boolean_idx" ;
    "cadmiumjdbc_resultset_update_boolean_name" ;
    "cadmiumjdbc_resultset_update_byte_idx" ;
    "cadmiumjdbc_resultset_update_byte_name" ;
    "cadmiumjdbc_resultset_update_bytes_idx" ;
    "cadmiumjdbc_resultset_update_bytes_name" ;
    "cadmiumjdbc_resultset_update_clob_idx" ;
    "cadmiumjdbc_resultset_update_clob_name" ;
    "cadmiumjdbc_resultset_update_date_idx" ;
    "cadmiumjdbc_resultset_update_date_name" ;
    "cadmiumjdbc_resultset_update_double_idx" ;
    "cadmiumjdbc_resultset_update_double_name" ;
    "cadmiumjdbc_resultset_update_float_idx" ;
    "cadmiumjdbc_resultset_update_float_name" ;
    "cadmiumjdbc_resultset_update_int_idx" ;
    "cadmiumjdbc_resultset_update_int_name" ;
    "cadmiumjdbc_resultset_update_long_idx" ;
    "cadmiumjdbc_resultset_update_long_name" ;
    "cadmiumjdbc_resultset_update_nclob_idx" ;
    "cadmiumjdbc_resultset_update_nclob_name" ;
    "cadmiumjdbc_resultset_update_nstring_idx" ;
    "cadmiumjdbc_resultset_update_nstring_name" ;
    "cadmiumjdbc_resultset_update_object_idx" ;
    "cadmiumjdbc_resultset_update_object_name" ;
    "cadmiumjdbc_resultset_update_object_scale_idx" ;
    "cadmiumjdbc_resultset_update_object_scale_name" ;
    "cadmiumjdbc_resultset_update_ref_idx" ;
    "cadmiumjdbc_resultset_update_ref_name" ;
    "cadmiumjdbc_resultset_update_row_id_idx" ;
    "cadmiumjdbc_resultset_update_row_id_name" ;
    "cadmiumjdbc_resultset_update_short_idx" ;
    "cadmiumjdbc_resultset_update_short_name" ;
    "cadmiumjdbc_resultset_update_sqlxml_idx" ;
    "cadmiumjdbc_resultset_update_sqlxml_name" ;
    "cadmiumjdbc_resultset_update_string_idx" ;
    "cadmiumjdbc_resultset_update_string_name" ;
    "cadmiumjdbc_resultset_update_time_idx" ;
    "cadmiumjdbc_resultset_update_time_name" ;
    "cadmiumjdbc_resultset_update_timestamp_idx" ;
    "cadmiumjdbc_resultset_update_timestamp_name" ;
    "cadmiumjdbc_resultset_update_row"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.RowIds", [
    "cadmiumjdbc_rowid_equals" ;
    "cadmiumjdbc_rowid_get_bytes" ;
    "cadmiumjdbc_rowid_hash_code" ;
    "cadmiumjdbc_rowid_to_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.SQLXMLs", [
    "cadmiumjdbc_sqlxml_free" ;
    "cadmiumjdbc_sqlxml_get_binary_stream" ;
    "cadmiumjdbc_sqlxml_get_string" ;
    "cadmiumjdbc_sqlxml_set_binary_stream" ;
    "cadmiumjdbc_sqlxml_set_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Savepoints", [
    "cadmiumjdbc_savepoint_get_savepoint_id" ;
    "cadmiumjdbc_savepoint_get_savepoint_name"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Statements", [
    "cadmiumjdbc_statement_add_batch" ;
    "cadmiumjdbc_statement_cancel" ;
    "cadmiumjdbc_statement_clear_batch" ;
    "cadmiumjdbc_statement_clear_warnings" ;
    "cadmiumjdbc_statement_close" ;
    "cadmiumjdbc_statement_execute" ;
    "cadmiumjdbc_statement_execute_autogen" ;
    "cadmiumjdbc_statement_execute_idx" ;
    "cadmiumjdbc_statement_execute_name" ;
    "cadmiumjdbc_statement_execute_batch" ;
    "cadmiumjdbc_statement_execute_query" ;
    "cadmiumjdbc_statement_execute_update" ;
    "cadmiumjdbc_statement_execute_update_autogen" ;
    "cadmiumjdbc_statement_execute_update_idx" ;
    "cadmiumjdbc_statement_execute_update_name" ;
    "cadmiumjdbc_statement_get_fetch_direction" ;
    "cadmiumjdbc_statement_get_fetch_size" ;
    "cadmiumjdbc_statement_get_generated_keys" ;
    "cadmiumjdbc_statement_get_max_field_size" ;
    "cadmiumjdbc_statement_get_max_rows" ;
    "cadmiumjdbc_statement_get_more_results" ;
    "cadmiumjdbc_statement_get_more_results_current" ;
    "cadmiumjdbc_statement_get_query_timeout" ;
    "cadmiumjdbc_statement_get_result_set" ;
    "cadmiumjdbc_statement_get_result_set_concurrency" ;
    "cadmiumjdbc_statement_get_result_set_holdability" ;
    "cadmiumjdbc_statement_get_result_set_type" ;
    "cadmiumjdbc_statement_get_update_count" ;
    "cadmiumjdbc_statement_get_warnings" ;
    "cadmiumjdbc_statement_is_closed" ;
    "cadmiumjdbc_statement_is_poolable" ;
    "cadmiumjdbc_statement_set_cursor_name" ;
    "cadmiumjdbc_statement_set_escape_processing" ;
    "cadmiumjdbc_statement_set_fetch_direction" ;
    "cadmiumjdbc_statement_set_fetch_size" ;
    "cadmiumjdbc_statement_set_max_field_size" ;
    "cadmiumjdbc_statement_set_max_rows" ;
    "cadmiumjdbc_statement_set_poolable" ;
    "cadmiumjdbc_statement_set_query_timeout"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Structs", [
    "cadmiumjdbc_struct_get_attributes" ;
    "cadmiumjdbc_struct_get_sql_type_name"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Times", [
    "cadmiumjdbc_time_make" ;
    "cadmiumjdbc_time_make_of_int64" ;
    "cadmiumjdbc_time_make_of_string" ;
    "cadmiumjdbc_time_get_time" ;
    "cadmiumjdbc_time_set_time" ;
    "cadmiumjdbc_time_to_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumjdbc.Timestamps", [
    "cadmiumjdbc_timestamp_make" ;
    "cadmiumjdbc_timestamp_make_of_int64" ;
    "cadmiumjdbc_timestamp_make_of_string" ;
    "cadmiumjdbc_timestamp_after" ;
    "cadmiumjdbc_timestamp_before" ;
    "cadmiumjdbc_timestamp_compare_to" ;
    "cadmiumjdbc_timestamp_get_nanos" ;
    "cadmiumjdbc_timestamp_get_time" ;
    "cadmiumjdbc_timestamp_set_nanos" ;
    "cadmiumjdbc_timestamp_set_time" ;
    "cadmiumjdbc_timestamp_to_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiummath.BigDecimals", [
    "cadmiummath_bigdecimal_get_constant" ;
    "cadmiummath_bigdecimal_make_of_big_integer" ;
    "cadmiummath_bigdecimal_make_of_big_integer_ctxt" ;
    "cadmiummath_bigdecimal_make_of_big_integer_scale" ;
    "cadmiummath_bigdecimal_make_of_big_integer_scale_ctxt" ;
    "cadmiummath_bigdecimal_make_of_float" ;
    "cadmiummath_bigdecimal_make_of_float_ctxt" ;
    "cadmiummath_bigdecimal_make_of_int32" ;
    "cadmiummath_bigdecimal_make_of_int32_ctxt" ;
    "cadmiummath_bigdecimal_make_of_int64" ;
    "cadmiummath_bigdecimal_make_of_int64_ctxt" ;
    "cadmiummath_bigdecimal_make_of_string" ;
    "cadmiummath_bigdecimal_make_of_string_ctxt" ;
    "cadmiummath_bigdecimal_abs" ;
    "cadmiummath_bigdecimal_abs_ctxt" ;
    "cadmiummath_bigdecimal_add" ;
    "cadmiummath_bigdecimal_add_ctxt" ;
    "cadmiummath_bigdecimal_byte_value_exact" ;
    "cadmiummath_bigdecimal_compare_to" ;
    "cadmiummath_bigdecimal_divide" ;
    "cadmiummath_bigdecimal_divide_scale_mode" ;
    "cadmiummath_bigdecimal_divide_ctxt" ;
    "cadmiummath_bigdecimal_divide_mode" ;
    "cadmiummath_bigdecimal_divide_and_remainder" ;
    "cadmiummath_bigdecimal_divide_and_remainder_ctxt" ;
    "cadmiummath_bigdecimal_divide_to_integral_value" ;
    "cadmiummath_bigdecimal_divide_to_integral_value_ctxt" ;
    "cadmiummath_bigdecimal_equals" ;
    "cadmiummath_bigdecimal_float_value" ;
    "cadmiummath_bigdecimal_hash_code" ;
    "cadmiummath_bigdecimal_int_value" ;
    "cadmiummath_bigdecimal_int_value_exact" ;
    "cadmiummath_bigdecimal_int32_value" ;
    "cadmiummath_bigdecimal_int32_value_exact" ;
    "cadmiummath_bigdecimal_int64_value" ;
    "cadmiummath_bigdecimal_int64_value_exact" ;
    "cadmiummath_bigdecimal_max" ;
    "cadmiummath_bigdecimal_min" ;
    "cadmiummath_bigdecimal_move_point_left" ;
    "cadmiummath_bigdecimal_move_point_right" ;
    "cadmiummath_bigdecimal_multiply" ;
    "cadmiummath_bigdecimal_multiply_ctxt" ;
    "cadmiummath_bigdecimal_negate" ;
    "cadmiummath_bigdecimal_negate_ctxt" ;
    "cadmiummath_bigdecimal_plus" ;
    "cadmiummath_bigdecimal_plus_ctxt" ;
    "cadmiummath_bigdecimal_pow" ;
    "cadmiummath_bigdecimal_pow_ctxt" ;
    "cadmiummath_bigdecimal_precision" ;
    "cadmiummath_bigdecimal_remainder" ;
    "cadmiummath_bigdecimal_remainder_ctxt" ;
    "cadmiummath_bigdecimal_round" ;
    "cadmiummath_bigdecimal_scale" ;
    "cadmiummath_bigdecimal_scale_by_power_of_ten" ;
    "cadmiummath_bigdecimal_set_scale" ;
    "cadmiummath_bigdecimal_set_scale_mode" ;
    "cadmiummath_bigdecimal_short_value_exact" ;
    "cadmiummath_bigdecimal_signum" ;
    "cadmiummath_bigdecimal_strip_trailing_zeros" ;
    "cadmiummath_bigdecimal_subtract" ;
    "cadmiummath_bigdecimal_subtract_ctxt" ;
    "cadmiummath_bigdecimal_to_big_integer" ;
    "cadmiummath_bigdecimal_to_big_integer_exact" ;
    "cadmiummath_bigdecimal_to_engineering_string" ;
    "cadmiummath_bigdecimal_to_plain_string" ;
    "cadmiummath_bigdecimal_to_string" ;
    "cadmiummath_bigdecimal_ulp" ;
    "cadmiummath_bigdecimal_unscaled_value"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiummath.BigIntegers", [
    "cadmiummath_biginteger_get_constant" ;
    "cadmiummath_biginteger_make_of_bytes" ;
    "cadmiummath_biginteger_make_of_signum_bytes" ;
    "cadmiummath_biginteger_make_prime_of_random" ;
    "cadmiummath_biginteger_make_of_random" ;
    "cadmiummath_biginteger_make_of_string" ;
    "cadmiummath_biginteger_make_of_string_radix" ;
    "cadmiummath_biginteger_make_of_int64" ;
    "cadmiummath_biginteger_abs" ;
    "cadmiummath_biginteger_add" ;
    "cadmiummath_biginteger_and_" ;
    "cadmiummath_biginteger_and_not" ;
    "cadmiummath_biginteger_bit_count" ;
    "cadmiummath_biginteger_bit_length" ;
    "cadmiummath_biginteger_clear_bit" ;
    "cadmiummath_biginteger_compare_to" ;
    "cadmiummath_biginteger_divide" ;
    "cadmiummath_biginteger_divide_and_remainder" ;
    "cadmiummath_biginteger_equals" ;
    "cadmiummath_biginteger_flip_bit" ;
    "cadmiummath_biginteger_float_value" ;
    "cadmiummath_biginteger_gcd" ;
    "cadmiummath_biginteger_get_lowest_set_bit" ;
    "cadmiummath_biginteger_hash_code" ;
    "cadmiummath_biginteger_int_value" ;
    "cadmiummath_biginteger_int32_value" ;
    "cadmiummath_biginteger_int64_value" ;
    "cadmiummath_biginteger_is_probably_prime" ;
    "cadmiummath_biginteger_max" ;
    "cadmiummath_biginteger_min" ;
    "cadmiummath_biginteger_mod_" ;
    "cadmiummath_biginteger_mod_inverse" ;
    "cadmiummath_biginteger_mod_pow" ;
    "cadmiummath_biginteger_multiply" ;
    "cadmiummath_biginteger_negate" ;
    "cadmiummath_biginteger_next_probable_prime" ;
    "cadmiummath_biginteger_not" ;
    "cadmiummath_biginteger_or_" ;
    "cadmiummath_biginteger_pow" ;
    "cadmiummath_biginteger_remainder" ;
    "cadmiummath_biginteger_set_bit" ;
    "cadmiummath_biginteger_shift_left" ;
    "cadmiummath_biginteger_shift_right" ;
    "cadmiummath_biginteger_signum" ;
    "cadmiummath_biginteger_subtract" ;
    "cadmiummath_biginteger_test_bit" ;
    "cadmiummath_biginteger_to_bytes" ;
    "cadmiummath_biginteger_to_string" ;
    "cadmiummath_biginteger_to_string_radix" ;
    "cadmiummath_biginteger_xor"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiummath.Floats", [
    "cadmiummath_float_get_e" ;
    "cadmiummath_float_get_pi" ;
    "cadmiummath_float_abs" ;
    "cadmiummath_float_acos" ;
    "cadmiummath_float_asin" ;
    "cadmiummath_float_atan" ;
    "cadmiummath_float_atan2" ;
    "cadmiummath_float_cbrt" ;
    "cadmiummath_float_ceil" ;
    "cadmiummath_float_copy_sign" ;
    "cadmiummath_float_cos" ;
    "cadmiummath_float_cosh" ;
    "cadmiummath_float_exp" ;
    "cadmiummath_float_exp_m1" ;
    "cadmiummath_float_floor" ;
    "cadmiummath_float_get_exponent" ;
    "cadmiummath_float_hypot" ;
    "cadmiummath_float_ieee_remainder" ;
    "cadmiummath_float_log" ;
    "cadmiummath_float_log10" ;
    "cadmiummath_float_log1p" ;
    "cadmiummath_float_max" ;
    "cadmiummath_float_min" ;
    "cadmiummath_float_next_after" ;
    "cadmiummath_float_next_up" ;
    "cadmiummath_float_pow" ;
    "cadmiummath_float_random" ;
    "cadmiummath_float_rint" ;
    "cadmiummath_float_round" ;
    "cadmiummath_float_scalb" ;
    "cadmiummath_float_signum" ;
    "cadmiummath_float_sin" ;
    "cadmiummath_float_sinh" ;
    "cadmiummath_float_sqrt" ;
    "cadmiummath_float_tan" ;
    "cadmiummath_float_tanh" ;
    "cadmiummath_float_to_degrees" ;
    "cadmiummath_float_to_radians" ;
    "cadmiummath_float_ulp"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiummath.Randoms", [
    "cadmiummath_random_make" ;
    "cadmiummath_random_make_of_int64" ;
    "cadmiummath_random_next_boolean" ;
    "cadmiummath_random_next_bytes" ;
    "cadmiummath_random_next_float" ;
    "cadmiummath_random_next_gaussian" ;
    "cadmiummath_random_next_int" ;
    "cadmiummath_random_next_int_n" ;
    "cadmiummath_random_next_int32" ;
    "cadmiummath_random_next_int32_n" ;
    "cadmiummath_random_next_int64" ;
    "cadmiummath_random_set_seed"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiummath.StrictFloats", [
    "cadmiummath_strictfloat_get_e" ;
    "cadmiummath_strictfloat_get_pi" ;
    "cadmiummath_strictfloat_abs" ;
    "cadmiummath_strictfloat_acos" ;
    "cadmiummath_strictfloat_asin" ;
    "cadmiummath_strictfloat_atan" ;
    "cadmiummath_strictfloat_atan2" ;
    "cadmiummath_strictfloat_cbrt" ;
    "cadmiummath_strictfloat_ceil" ;
    "cadmiummath_strictfloat_copy_sign" ;
    "cadmiummath_strictfloat_cos" ;
    "cadmiummath_strictfloat_cosh" ;
    "cadmiummath_strictfloat_exp" ;
    "cadmiummath_strictfloat_exp_m1" ;
    "cadmiummath_strictfloat_floor" ;
    "cadmiummath_strictfloat_get_exponent" ;
    "cadmiummath_strictfloat_hypot" ;
    "cadmiummath_strictfloat_ieee_remainder" ;
    "cadmiummath_strictfloat_log" ;
    "cadmiummath_strictfloat_log10" ;
    "cadmiummath_strictfloat_log1p" ;
    "cadmiummath_strictfloat_max" ;
    "cadmiummath_strictfloat_min" ;
    "cadmiummath_strictfloat_next_after" ;
    "cadmiummath_strictfloat_next_up" ;
    "cadmiummath_strictfloat_pow" ;
    "cadmiummath_strictfloat_random" ;
    "cadmiummath_strictfloat_rint" ;
    "cadmiummath_strictfloat_round" ;
    "cadmiummath_strictfloat_scalb" ;
    "cadmiummath_strictfloat_signum" ;
    "cadmiummath_strictfloat_sin" ;
    "cadmiummath_strictfloat_sinh" ;
    "cadmiummath_strictfloat_sqrt" ;
    "cadmiummath_strictfloat_tan" ;
    "cadmiummath_strictfloat_tanh" ;
    "cadmiummath_strictfloat_to_degrees" ;
    "cadmiummath_strictfloat_to_radians" ;
    "cadmiummath_strictfloat_ulp"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumservlet.Servlets", [
    "cadmiumservlet_cookie_make" ;
    "cadmiumservlet_cookie_clone" ;
    "cadmiumservlet_cookie_get_comment" ;
    "cadmiumservlet_cookie_get_domain" ;
    "cadmiumservlet_cookie_get_max_age" ;
    "cadmiumservlet_cookie_get_name" ;
    "cadmiumservlet_cookie_get_path" ;
    "cadmiumservlet_cookie_get_secure" ;
    "cadmiumservlet_cookie_get_value" ;
    "cadmiumservlet_cookie_get_version" ;
    "cadmiumservlet_cookie_set_comment" ;
    "cadmiumservlet_cookie_set_domain" ;
    "cadmiumservlet_cookie_set_max_age" ;
    "cadmiumservlet_cookie_set_path" ;
    "cadmiumservlet_cookie_set_secure" ;
    "cadmiumservlet_cookie_set_value" ;
    "cadmiumservlet_cookie_set_version" ;
    "cadmiumservlet_context_get_attribute" ;
    "cadmiumservlet_context_get_attribute_names" ;
    "cadmiumservlet_context_get_context" ;
    "cadmiumservlet_context_get_context_path" ;
    "cadmiumservlet_context_get_init_parameter" ;
    "cadmiumservlet_context_get_init_parameter_names" ;
    "cadmiumservlet_context_get_major_version" ;
    "cadmiumservlet_context_get_mime_type" ;
    "cadmiumservlet_context_get_minor_version" ;
    "cadmiumservlet_context_get_real_path" ;
    "cadmiumservlet_context_get_resource_as_url" ;
    "cadmiumservlet_context_get_resource_as_stream" ;
    "cadmiumservlet_context_get_resource_paths" ;
    "cadmiumservlet_context_get_server_info" ;
    "cadmiumservlet_context_get_servlet_context_name" ;
    "cadmiumservlet_context_log" ;
    "cadmiumservlet_context_log_exn" ;
    "cadmiumservlet_context_remove_attribute" ;
    "cadmiumservlet_context_set_attribute" ;
    "cadmiumservlet_session_get_attribute" ;
    "cadmiumservlet_session_get_attribute_names" ;
    "cadmiumservlet_session_get_creation_time" ;
    "cadmiumservlet_session_get_id" ;
    "cadmiumservlet_session_get_last_accessed_time" ;
    "cadmiumservlet_session_get_max_inactive_interval" ;
    "cadmiumservlet_session_get_servlet_context" ;
    "cadmiumservlet_session_invalidate" ;
    "cadmiumservlet_session_is_new" ;
    "cadmiumservlet_session_remove_attribute" ;
    "cadmiumservlet_session_set_attribute" ;
    "cadmiumservlet_session_set_max_inactive_interval" ;
    "cadmiumservlet_request_get_attribute" ;
    "cadmiumservlet_request_get_attribute_names" ;
    "cadmiumservlet_request_get_auth_type" ;
    "cadmiumservlet_request_get_character_encoding" ;
    "cadmiumservlet_request_get_content_length" ;
    "cadmiumservlet_request_get_content_type" ;
    "cadmiumservlet_request_get_context_path" ;
    "cadmiumservlet_request_get_cookies" ;
    "cadmiumservlet_request_get_date_header" ;
    "cadmiumservlet_request_get_header" ;
    "cadmiumservlet_request_get_header_names" ;
    "cadmiumservlet_request_get_headers" ;
    "cadmiumservlet_request_get_input_stream" ;
    "cadmiumservlet_request_get_int_header" ;
    "cadmiumservlet_request_get_local_addr" ;
    "cadmiumservlet_request_get_locale" ;
    "cadmiumservlet_request_get_locales" ;
    "cadmiumservlet_request_get_local_name" ;
    "cadmiumservlet_request_get_local_port" ;
    "cadmiumservlet_request_get_method" ;
    "cadmiumservlet_request_get_parameter" ;
    "cadmiumservlet_request_get_parameter_list" ;
    "cadmiumservlet_request_get_parameter_names" ;
    "cadmiumservlet_request_get_parameter_values" ;
    "cadmiumservlet_request_get_path_info" ;
    "cadmiumservlet_request_get_path_translated" ;
    "cadmiumservlet_request_get_protocol" ;
    "cadmiumservlet_request_get_query_string" ;
    "cadmiumservlet_request_get_remote_addr" ;
    "cadmiumservlet_request_get_remote_host" ;
    "cadmiumservlet_request_get_remote_port" ;
    "cadmiumservlet_request_get_remote_user" ;
    "cadmiumservlet_request_get_requested_session_id" ;
    "cadmiumservlet_request_get_request_uri" ;
    "cadmiumservlet_request_get_request_url" ;
    "cadmiumservlet_request_get_scheme" ;
    "cadmiumservlet_request_get_server_name" ;
    "cadmiumservlet_request_get_server_port" ;
    "cadmiumservlet_request_get_servlet_path" ;
    "cadmiumservlet_request_get_session" ;
    "cadmiumservlet_request_get_session_creating" ;
    "cadmiumservlet_request_is_requested_session_id_from_cookie" ;
    "cadmiumservlet_request_is_requested_session_id_from_url" ;
    "cadmiumservlet_request_is_requested_session_id_valid" ;
    "cadmiumservlet_request_is_secure" ;
    "cadmiumservlet_request_is_user_in_role" ;
    "cadmiumservlet_request_remove_attribute" ;
    "cadmiumservlet_request_set_attribute" ;
    "cadmiumservlet_request_set_character_encoding" ;
    "cadmiumservlet_response_add_cookie" ;
    "cadmiumservlet_response_add_date_header" ;
    "cadmiumservlet_response_add_header" ;
    "cadmiumservlet_response_add_int_header" ;
    "cadmiumservlet_response_contains_header" ;
    "cadmiumservlet_response_encode_redirect_url" ;
    "cadmiumservlet_response_encode_url" ;
    "cadmiumservlet_response_flush_buffer" ;
    "cadmiumservlet_response_get_buffer_size" ;
    "cadmiumservlet_response_get_character_encoding" ;
    "cadmiumservlet_response_get_content_type" ;
    "cadmiumservlet_response_get_locale" ;
    "cadmiumservlet_response_get_output_stream" ;
    "cadmiumservlet_response_is_committed" ;
    "cadmiumservlet_response_reset" ;
    "cadmiumservlet_response_reset_buffer" ;
    "cadmiumservlet_response_send_error" ;
    "cadmiumservlet_response_send_error_msg" ;
    "cadmiumservlet_response_send_redirect" ;
    "cadmiumservlet_response_set_buffer_size" ;
    "cadmiumservlet_response_set_character_encoding" ;
    "cadmiumservlet_response_set_content_length" ;
    "cadmiumservlet_response_set_content_type" ;
    "cadmiumservlet_response_set_date_header" ;
    "cadmiumservlet_response_set_header" ;
    "cadmiumservlet_response_set_int_header" ;
    "cadmiumservlet_response_set_locale" ;
    "cadmiumservlet_response_set_status" ;
    "cadmiumservlet_servlet_get_init_parameter" ;
    "cadmiumservlet_servlet_get_init_parameter_names" ;
    "cadmiumservlet_servlet_get_servlet_context" ;
    "cadmiumservlet_servlet_get_servlet_name" ;
    "cadmiumservlet_servlet_log" ;
    "cadmiumservlet_servlet_log_exn"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumswixml.SwiXml", [
    "cadmiumswixml_is_debug_mode" ;
    "cadmiumswixml_is_macosx" ;
    "cadmiumswixml_is_macosx_supported" ;
    "cadmiumswixml_set_macosx_support" ;
    "cadmiumswixml_set_default_locale" ;
    "cadmiumswixml_set_resource_bundle_name" ;
    "cadmiumswixml_set_app_frame" ;
    "cadmiumswixml_get_app_frame" ;
    "cadmiumswixml_make_engine" ;
    "cadmiumswixml_make_engine_from_resource" ;
    "cadmiumswixml_make_engine_from_client" ;
    "cadmiumswixml_get_client" ;
    "cadmiumswixml_set_classloader" ;
    "cadmiumswixml_get_classloader" ;
    "cadmiumswixml_set_locale" ;
    "cadmiumswixml_set_resource_bundle" ;
    "cadmiumswixml_render_from_file" ;
    "cadmiumswixml_render_from_url" ;
    "cadmiumswixml_render_from_resource" ;
    "cadmiumswixml_render_from_channel" ;
    "cadmiumswixml_insert_from_url" ;
    "cadmiumswixml_insert_from_resource" ;
    "cadmiumswixml_insert_from_channel" ;
    "cadmiumswixml_get_root_component" ;
    "cadmiumswixml_get_id_map" ;
    "cadmiumswixml_clear_id_map" ;
    "cadmiumswixml_get_all_components" ;
    "cadmiumswixml_get_descendants" ;
    "cadmiumswixml_find" ;
    "cadmiumswixml_forget" ;
    "cadmiumswixml_cleanup" ;
    "cadmiumswixml_set_action_listener"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumxml.Documents", [
    "cadmiumxml_document_get_document_element" ;
    "cadmiumxml_document_get_xml_encoding" ;
    "cadmiumxml_document_set_strict_error_checking" ;
    "cadmiumxml_document_is_strict_error_checking" ;
    "cadmiumxml_document_set_xml_standalone" ;
    "cadmiumxml_document_is_xml_standalone" ;
    "cadmiumxml_document_set_xml_version" ;
    "cadmiumxml_document_get_xml_version" ;
    "cadmiumxml_document_normalize_document" ;
    "cadmiumxml_document_get_attr_parent" ;
    "cadmiumxml_document_get_attr_name" ;
    "cadmiumxml_document_get_attr_value" ;
    "cadmiumxml_document_set_attr_value" ;
    "cadmiumxml_document_get_cdata_parent" ;
    "cadmiumxml_document_get_cdata" ;
    "cadmiumxml_document_set_cdata" ;
    "cadmiumxml_document_get_comment_parent" ;
    "cadmiumxml_document_get_comment" ;
    "cadmiumxml_document_set_comment" ;
    "cadmiumxml_document_get_document_type_parent" ;
    "cadmiumxml_document_get_document_type_name" ;
    "cadmiumxml_document_get_document_type_public_id" ;
    "cadmiumxml_document_get_document_type_system_id" ;
    "cadmiumxml_document_get_element_parent" ;
    "cadmiumxml_document_get_attribute" ;
    "cadmiumxml_document_get_attribute_ns" ;
    "cadmiumxml_document_get_attribute_node" ;
    "cadmiumxml_document_get_attribute_node_ns" ;
    "cadmiumxml_document_get_elements_by_tag_name" ;
    "cadmiumxml_document_get_elements_by_tag_name_ns" ;
    "cadmiumxml_document_get_tag_name" ;
    "cadmiumxml_document_has_attribute" ;
    "cadmiumxml_document_has_attribute_ns" ;
    "cadmiumxml_document_remove_attribute" ;
    "cadmiumxml_document_remove_attribute_ns" ;
    "cadmiumxml_document_remove_attribute_node" ;
    "cadmiumxml_document_set_attribute" ;
    "cadmiumxml_document_set_attribute_ns" ;
    "cadmiumxml_document_get_entity_parent" ;
    "cadmiumxml_document_get_entity_notation_name" ;
    "cadmiumxml_document_get_entity_public_id" ;
    "cadmiumxml_document_get_entity_system_id" ;
    "cadmiumxml_document_get_notation_parent" ;
    "cadmiumxml_document_get_notation_public_id" ;
    "cadmiumxml_document_get_notation_system_id" ;
    "cadmiumxml_document_get_processing_instruction_parent" ;
    "cadmiumxml_document_get_processing_instruction_target" ;
    "cadmiumxml_document_get_processing_instruction_data" ;
    "cadmiumxml_document_set_processing_instruction_data" ;
    "cadmiumxml_document_get_text_parent" ;
    "cadmiumxml_document_get_text" ;
    "cadmiumxml_document_set_text" ;
    "cadmiumxml_document_get_parent" ;
    "cadmiumxml_document_get_document" ;
    "cadmiumxml_document_append_attr_child" ;
    "cadmiumxml_document_append_attr_ns_child" ;
    "cadmiumxml_document_append_cdata_child" ;
    "cadmiumxml_document_append_comment_child" ;
    "cadmiumxml_document_append_element_child" ;
    "cadmiumxml_document_append_element_ns_child" ;
    "cadmiumxml_document_append_entity_reference_child" ;
    "cadmiumxml_document_insert_attr_child_before" ;
    "cadmiumxml_document_insert_attr_ns_child_before" ;
    "cadmiumxml_document_insert_cdata_child_before" ;
    "cadmiumxml_document_insert_comment_child_before" ;
    "cadmiumxml_document_insert_element_child_before" ;
    "cadmiumxml_document_insert_element_ns_child_before" ;
    "cadmiumxml_document_insert_entity_reference_child_before" ;
    "cadmiumxml_document_get_child_nodes" ;
    "cadmiumxml_document_get_first_child" ;
    "cadmiumxml_document_get_last_child" ;
    "cadmiumxml_document_get_previous_sibling" ;
    "cadmiumxml_document_get_next_sibling" ;
    "cadmiumxml_document_remove_child" ;
    "cadmiumxml_document_replace_child" ;
    "cadmiumxml_documentfactory_make" ;
    "cadmiumxml_documentfactory_set_attribute" ;
    "cadmiumxml_documentfactory_get_attribute" ;
    "cadmiumxml_documentfactory_set_feature" ;
    "cadmiumxml_documentfactory_get_feature" ;
    "cadmiumxml_documentfactory_set_schema" ;
    "cadmiumxml_documentfactory_get_schema" ;
    "cadmiumxml_documentfactory_set_coalescing" ;
    "cadmiumxml_documentfactory_is_coalescing" ;
    "cadmiumxml_documentfactory_set_expand_entity_references" ;
    "cadmiumxml_documentfactory_is_expand_entity_references" ;
    "cadmiumxml_documentfactory_set_ignoring_comments" ;
    "cadmiumxml_documentfactory_is_ignoring_comments" ;
    "cadmiumxml_documentfactory_set_ignoring_element_content_whitespace" ;
    "cadmiumxml_documentfactory_is_ignoring_element_content_whitespace" ;
    "cadmiumxml_documentfactory_set_namespace_aware" ;
    "cadmiumxml_documentfactory_is_namespace_aware" ;
    "cadmiumxml_documentfactory_set_validating" ;
    "cadmiumxml_documentfactory_is_validating" ;
    "cadmiumxml_documentfactory_set_xinclude_aware" ;
    "cadmiumxml_documentfactory_is_xinclude_aware" ;
    "cadmiumxml_documentbuilder_make" ;
    "cadmiumxml_documentbuilder_get_schema" ;
    "cadmiumxml_documentbuilder_is_namespace_aware" ;
    "cadmiumxml_documentbuilder_is_validating" ;
    "cadmiumxml_documentbuilder_is_xinclude_aware" ;
    "cadmiumxml_documentbuilder_new_document" ;
    "cadmiumxml_documentbuilder_parse_file" ;
    "cadmiumxml_documentbuilder_parse_channel" ;
    "cadmiumxml_documentbuilder_reset"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumxml.SAX", [
    "cadmiumxml_saxfactory_make" ;
    "cadmiumxml_saxfactory_set_feature" ;
    "cadmiumxml_saxfactory_get_feature" ;
    "cadmiumxml_saxfactory_set_schema" ;
    "cadmiumxml_saxfactory_get_schema" ;
    "cadmiumxml_saxfactory_set_namespace_aware" ;
    "cadmiumxml_saxfactory_is_namespace_aware" ;
    "cadmiumxml_saxfactory_set_validating" ;
    "cadmiumxml_saxfactory_is_validating" ;
    "cadmiumxml_saxfactory_set_xinclude_aware" ;
    "cadmiumxml_saxfactory_is_xinclude_aware" ;
    "cadmiumxml_sax_make" ;
    "cadmiumxml_sax_set_property" ;
    "cadmiumxml_sax_get_property" ;
    "cadmiumxml_sax_get_schema" ;
    "cadmiumxml_sax_is_namespace_aware" ;
    "cadmiumxml_sax_is_validating" ;
    "cadmiumxml_sax_is_xinclude_aware" ;
    "cadmiumxml_sax_parse_file" ;
    "cadmiumxml_sax_parse_channel" ;
    "cadmiumxml_sax_reset"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumxml.Schemas", [
    "cadmiumxml_schemafactory_make" ;
    "cadmiumxml_schemafactory_set_feature" ;
    "cadmiumxml_schemafactory_get_feature" ;
    "cadmiumxml_schemafactory_set_property" ;
    "cadmiumxml_schemafactory_get_property" ;
    "cadmiumxml_schema_from_file" ;
    "cadmiumxml_schema_from_url"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumxml.Transformers", [
    "cadmiumxml_transformerfactory_make" ;
    "cadmiumxml_transformerfactory_set_attribute" ;
    "cadmiumxml_transformerfactory_get_attribute" ;
    "cadmiumxml_transformerfactory_set_feature" ;
    "cadmiumxml_transformerfactory_get_feature" ;
    "cadmiumxml_transformer_make_id" ;
    "cadmiumxml_transformer_make_from_file" ;
    "cadmiumxml_transformer_make_from_channel" ;
    "cadmiumxml_transformer_clear_parameters" ;
    "cadmiumxml_transformer_set_parameter" ;
    "cadmiumxml_transformer_get_parameter" ;
    "cadmiumxml_transformer_set_output_property" ;
    "cadmiumxml_transformer_get_output_property" ;
    "cadmiumxml_transformer_transform_file_to_file" ;
    "cadmiumxml_transformer_transform_file_to_channel" ;
    "cadmiumxml_transformer_transform_channel_to_file" ;
    "cadmiumxml_transformer_transform_channel_to_channel" ;
    "cadmiumxml_transformer_reset"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumxml.Trees", [
    "cadmiumxml_tree_of_document"
  ]) ;
  ("fr.x9c.cadmium.primitives.cadmiumxml.XPath", [
    "cadmiumxml_xpathfactory_make" ;
    "cadmiumxml_xpathfactory_set_feature" ;
    "cadmiumxml_xpathfactory_get_feature" ;
    "cadmiumxml_xpath_make" ;
    "cadmiumxml_xpath_compile" ;
    "cadmiumxml_xpath_evaluate_file" ;
    "cadmiumxml_xpath_evaluate_channel" ;
    "cadmiumxml_xpath_evaluate_document" ;
    "cadmiumxml_xpath_expr_evaluate_file" ;
    "cadmiumxml_xpath_expr_evaluate_channel" ;
    "cadmiumxml_xpath_expr_evaluate_document" ;
    "cadmiumxml_xpath_reset"
  ]) ;
  ("fr.x9c.cadmium.primitives.bigarray.BigArray", [
    "caml_ba_init" ;
    "caml_ba_create" ;
    "caml_ba_get_1" ;
    "caml_ba_get_2" ;
    "caml_ba_get_3" ;
    "caml_ba_get_generic" ;
    "caml_ba_set_1" ;
    "caml_ba_set_2" ;
    "caml_ba_set_3" ;
    "caml_ba_set_generic" ;
    "caml_ba_num_dims" ;
    "caml_ba_dim" ;
    "caml_ba_kind" ;
    "caml_ba_layout" ;
    "caml_ba_sub" ;
    "caml_ba_slice" ;
    "caml_ba_blit" ;
    "caml_ba_fill" ;
    "caml_ba_map_file" ;
    "caml_ba_map_file_bytecode" ;
    "caml_ba_reshape"
  ]) ;
  ("fr.x9c.cadmium.primitives.dbm.Cldbm", [
    "caml_dbm_open" ;
    "caml_dbm_close" ;
    "caml_dbm_fetch" ;
    "caml_dbm_insert" ;
    "caml_dbm_replace" ;
    "caml_dbm_delete" ;
    "caml_dbm_firstkey" ;
    "caml_dbm_nextkey"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Color", [
    "caml_gr_set_color"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Draw", [
    "caml_gr_plot" ;
    "caml_gr_moveto" ;
    "caml_gr_current_x" ;
    "caml_gr_current_y" ;
    "caml_gr_lineto" ;
    "caml_gr_draw_rect" ;
    "caml_gr_draw_arc" ;
    "caml_gr_draw_arc_nat" ;
    "caml_gr_set_line_width"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.DumpImage", [
    "caml_gr_dump_image"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Events", [
    "caml_gr_wait_event"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Fill", [
    "caml_gr_fill_rect" ;
    "caml_gr_fill_poly" ;
    "caml_gr_fill_arc" ;
    "caml_gr_fill_arc_nat"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Image", [
    "caml_gr_create_image" ;
    "caml_gr_blit_image" ;
    "caml_gr_draw_image"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.MakeImage", [
    "caml_gr_make_image"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Open", [
    "caml_gr_open_graph" ;
    "caml_gr_window_id" ;
    "caml_gr_close_graph" ;
    "caml_gr_set_window_title" ;
    "caml_gr_resize_window" ;
    "caml_gr_clear_graph" ;
    "caml_gr_size_x" ;
    "caml_gr_size_y" ;
    "caml_gr_display_mode" ;
    "caml_gr_remember_mode" ;
    "caml_gr_synchronize" ;
    "caml_gr_sigio_signal" ;
    "caml_gr_sigio_handler"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.PointCol", [
    "caml_gr_point_color"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Sound", [
    "caml_gr_sound"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.SubWindow", [
    "caml_gr_open_subwindow" ;
    "caml_gr_close_subwindow"
  ]) ;
  ("fr.x9c.cadmium.primitives.graph.Text", [
    "caml_gr_set_font" ;
    "caml_gr_set_text_size" ;
    "caml_gr_draw_char" ;
    "caml_gr_draw_string" ;
    "caml_gr_text_size"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkCaml", [
    "camltk_return" ;
    "camltk_init"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkEval", [
    "camltk_tcl_eval" ;
    "camltk_tcl_direct_eval"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkEvent", [
    "camltk_tk_mainloop" ;
    "camltk_dooneevent"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkFile", [
    "camltk_add_file_input" ;
    "camltk_rem_file_input" ;
    "camltk_add_file_output" ;
    "camltk_rem_file_output"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkImg", [
    "camltk_getimgdata" ;
    "camltk_setimgdata_native" ;
    "camltk_setimgdata_bytecode"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkMain", [
    "camltk_opentk" ;
    "camltk_finalize"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkMisc", [
    "camltk_splitlist"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkTimer", [
    "camltk_add_timer" ;
    "camltk_rem_timer"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkVar", [
    "camltk_getvar" ;
    "camltk_setvar" ;
    "camltk_trace_var" ;
    "camltk_untrace_var"
  ]) ;
  ("fr.x9c.cadmium.primitives.labltk.CltkWait", [
    "camltk_wait_vis" ;
    "camltk_wait_des"
  ]) ;
  ("fr.x9c.cadmium.primitives.num.Nat", [
    "initialize_nat" ;
    "create_nat" ;
    "length_nat" ;
    "set_to_zero_nat" ;
    "blit_nat" ;
    "set_digit_nat" ;
    "nth_digit_nat" ;
    "set_digit_nat_native" ;
    "nth_digit_nat_native" ;
    "num_digits_nat" ;
    "num_leading_zero_bits_in_digit" ;
    "is_digit_int" ;
    "is_digit_zero" ;
    "is_digit_normalized" ;
    "is_digit_odd" ;
    "incr_nat" ;
    "add_nat" ;
    "add_nat_native" ;
    "complement_nat" ;
    "decr_nat" ;
    "sub_nat" ;
    "sub_nat_native" ;
    "mult_digit_nat" ;
    "mult_digit_nat_native" ;
    "mult_nat" ;
    "mult_nat_native" ;
    "square_nat" ;
    "square_nat_native" ;
    "shift_left_nat" ;
    "shift_left_nat_native" ;
    "div_digit_nat" ;
    "div_digit_nat_native" ;
    "div_nat" ;
    "div_nat_native" ;
    "shift_right_nat" ;
    "shift_right_nat_native" ;
    "compare_digits_nat" ;
    "compare_nat" ;
    "compare_nat_native" ;
    "land_digit_nat" ;
    "lor_digit_nat" ;
    "lxor_digit_nat"
  ]) ;
  ("fr.x9c.cadmium.primitives.str.Str", [
    "re_string_match" ;
    "re_partial_match" ;
    "re_search_forward" ;
    "re_search_backward" ;
    "re_replacement_text"
  ]) ;
  ("fr.x9c.cadmium.primitives.systhreads.Threads", [
    "caml_thread_initialize" ;
    "caml_thread_new" ;
    "caml_thread_self" ;
    "caml_thread_id" ;
    "caml_thread_uncaught_exception" ;
    "caml_thread_exit" ;
    "caml_thread_yield" ;
    "caml_thread_join" ;
    "caml_mutex_new" ;
    "caml_mutex_lock" ;
    "caml_mutex_unlock" ;
    "caml_mutex_try_lock" ;
    "caml_condition_new" ;
    "caml_condition_wait" ;
    "caml_condition_signal" ;
    "caml_condition_broadcast" ;
    "caml_thread_sigmask" ;
    "caml_wait_signal"
  ]) ;
  ("fr.x9c.cadmium.primitives.threads.Threads", [
    "thread_initialize" ;
    "thread_initialize_preemption" ;
    "thread_new" ;
    "thread_id" ;
    "thread_yield" ;
    "thread_request_reschedule" ;
    "thread_sleep" ;
    "thread_wait_read" ;
    "thread_wait_write" ;
    "thread_wait_timed_read" ;
    "thread_wait_timed_write" ;
    "thread_select" ;
    "thread_inchan_ready" ;
    "thread_outchan_ready" ;
    "thread_delay" ;
    "thread_join" ;
    "thread_wait_pid" ;
    "thread_wakeup" ;
    "thread_self" ;
    "thread_kill" ;
    "thread_uncaught_exception"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Accept", [
    "unix_accept"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Access", [
    "unix_access"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Addrofstr", [
    "unix_inet_addr_of_string"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Alarm", [
    "unix_alarm"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Bind", [
    "unix_bind"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Chdir", [
    "unix_chdir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Chmod", [
    "unix_chmod"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Chown", [
    "unix_chown"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Chroot", [
    "unix_chroot"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Close", [
    "unix_close"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Closedir", [
    "unix_closedir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Connect", [
    "unix_connect"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Dup", [
    "unix_dup"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Dup2", [
    "unix_dup2"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Envir", [
    "unix_environment"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Errmsg", [
    "unix_error_message"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Execv", [
    "unix_execv"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Execve", [
    "unix_execve"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Execvp", [
    "unix_execvp" ;
    "unix_execvpe"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Exit", [
    "unix_exit"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Fchmod", [
    "unix_fchmod"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Fchown", [
    "unix_fchown"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Fcntl", [
    "unix_set_nonblock" ;
    "unix_clear_nonblock" ;
    "unix_set_close_on_exec" ;
    "unix_clear_close_on_exec"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Fork", [
    "unix_fork"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Ftruncate", [
    "unix_ftruncate" ;
    "unix_ftruncate_64"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getaddrinfo", [
    "unix_getaddrinfo"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getcwd", [
    "unix_getcwd"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getegid", [
    "unix_getegid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Geteuid", [
    "unix_geteuid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getgid", [
    "unix_getgid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getgr", [
    "unix_getgrnam" ;
    "unix_getgrgid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getgroups", [
    "unix_getgroups"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Gethost", [
    "unix_gethostbyaddr" ;
    "unix_gethostbyname"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Gethostname", [
    "unix_gethostname"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getlogin", [
    "unix_getlogin"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getnameinfo", [
    "unix_getnameinfo"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getpeername", [
    "unix_getpeername"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getpid", [
    "unix_getpid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getppid", [
    "unix_getppid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getproto", [
    "unix_getprotobyname" ;
    "unix_getprotobynumber"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getpw", [
    "unix_getpwnam" ;
    "unix_getpwuid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getserv", [
    "unix_getservbyname" ;
    "unix_getservbyport"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getsockname", [
    "unix_getsockname"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Gettimeofday", [
    "unix_gettimeofday"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Getuid", [
    "unix_getuid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Gmtime", [
    "unix_gmtime" ;
    "unix_localtime" ;
    "unix_mktime"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Isatty", [
    "unix_isatty"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Itimer", [
    "unix_setitimer" ;
    "unix_getitimer"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Kill", [
    "unix_kill"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Link", [
    "unix_link"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Listen", [
    "unix_listen"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Lockf", [
    "unix_lockf"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Lseek", [
    "unix_lseek" ;
    "unix_lseek_64"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Mkdir", [
    "unix_mkdir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Mkfifo", [
    "unix_mkfifo"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Nice", [
    "unix_nice"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Open", [
    "unix_open"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Opendir", [
    "unix_opendir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Pipe", [
    "unix_pipe"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Putenv", [
    "unix_putenv"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Read", [
    "unix_read"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Readdir", [
    "unix_readdir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Readlink", [
    "unix_readlink"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Rename", [
    "unix_rename"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Rewinddir", [
    "unix_rewinddir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Rmdir", [
    "unix_rmdir"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Select", [
    "unix_select"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Sendrecv", [
    "unix_recv" ;
    "unix_send" ;
    "unix_recvfrom" ;
    "unix_sendto" ;
    "unix_sendto_native"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Setgid", [
    "unix_setgid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Setsid", [
    "unix_setsid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Setuid", [
    "unix_setuid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Shutdown", [
    "unix_shutdown"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Signals", [
    "unix_sigprocmask" ;
    "unix_sigpending" ;
    "unix_sigsuspend"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Sleep", [
    "unix_sleep"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Socket", [
    "unix_socket"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Socketpair", [
    "unix_socketpair"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Sockopt", [
    "unix_getsockopt" ;
    "unix_setsockopt"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Stat", [
    "unix_stat" ;
    "unix_lstat" ;
    "unix_fstat" ;
    "unix_stat_64" ;
    "unix_lstat_64" ;
    "unix_fstat_64"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Strofaddr", [
    "unix_string_of_inet_addr"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Symlink", [
    "unix_symlink"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Termios", [
    "unix_tcgetattr" ;
    "unix_tcsetattr" ;
    "unix_tcsendbreak" ;
    "unix_tcdrain" ;
    "unix_tcflush" ;
    "unix_tcflow"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Time", [
    "unix_time"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Times", [
    "unix_times"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Truncate", [
    "unix_truncate" ;
    "unix_truncate_64"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Umask", [
    "unix_umask"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Unlink", [
    "unix_unlink"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Utimes", [
    "unix_utimes"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Wait", [
    "unix_wait" ;
    "unix_waitpid"
  ]) ;
  ("fr.x9c.cadmium.primitives.unix.Write", [
    "unix_write" ;
    "unix_single_write"
  ]) ;
  ("fr.x9c.cadmium.primitives.zip.Zlib", [
    "camlzip_deflateInit" ;
    "camlzip_deflate" ;
    "camlzip_deflate_bytecode" ;
    "camlzip_deflateEnd" ;
    "camlzip_inflateInit" ;
    "camlzip_inflate" ;
    "camlzip_inflate_bytecode" ;
    "camlzip_inflateEnd" ;
    "camlzip_update_crc32"
  ]) ;
  ("fr.x9c.cadmium.primitives.dbm.Cldbm", [
    "caml_dbm_open" ;
    "caml_dbm_close" ;
    "caml_dbm_fetch" ;
    "caml_dbm_insert" ;
    "caml_dbm_replace" ;
    "caml_dbm_delete" ;
    "caml_dbm_firstkey" ;
    "caml_dbm_nextkey"
  ]) ;
]

let table = Hashtbl.create 17

let is_runtime_visible_annot_with_class cls = function
  | `RuntimeVisibleAnnotations l ->
      List.exists
        (fun (n, _) ->
          utf8_eq
            (Name.external_utf8_for_class n)
            (utf8_of_string cls))
        l
  | _ -> false

let is_primitive =
  is_runtime_visible_annot_with_class "fr.x9c.cadmium.kernel.Primitive"

let is_primitive_provider =
  is_runtime_visible_annot_with_class "fr.x9c.cadmium.kernel.PrimitiveProvider"

let visit class_loader s =
  let cd =
    try
      ClassLoader.find class_loader (utf8_of_string s)
    with
    | ClassLoader.Exception (ClassLoader.Unable_to_load (_, v)) ->
	raise (Error (Undefined_class (s, v)))
    | ClassLoader.Exception _ -> raise (Error (Undefined_class (s, None))) in
  let cls = Name.external_utf8_for_class cd.ClassDefinition.name in
  let class_name = string_of_utf8 cls in
  let meths = ref [] in
  if List.exists is_primitive_provider cd.ClassDefinition.attributes then
    List.iter
      (function
        | Method.Regular (_, name, _, attributes) ->
            if List.exists is_primitive attributes then
              let method_name = string_of_utf8 (Name.utf8_for_method name) in
              (if !Jclflags.print_primitives then meths := method_name :: !meths);
              Hashtbl.add table method_name class_name
        | Method.Constructor _ | Method.Initializer _ -> ())
      cd.ClassDefinition.methods;
  if !Jclflags.print_primitives then begin
    Printf.printf "  (\"%s\", [\n    %s\n  ]) ;\n"
      s
      (String.concat
         " ;\n    "
         (List.rev_map (fun s -> "\"" ^ s ^ "\"") !meths))
  end

let initialized = ref false

let init () =
  if not !initialized then begin
    let cp = String.concat ":" (List.rev !Jclflags.class_path) in
    let class_path = ClassPath.make_of_string cp in
    let class_loader = ClassLoader.make class_path in
    if !Jclflags.no_builtin then
      List.iter
        (visit class_loader)
        all_classes
    else
      List.iter
        (fun (cls, meths) ->
          List.iter
            (fun meth_name ->
              Hashtbl.add table meth_name cls)
            meths)
        builtin;
    List.iter
      (visit class_loader)
      (List.rev !Jclflags.additional_providers);
    initialized := true
  end

let get_class name =
  init ();
  try
    Hashtbl.find table name
  with Not_found -> raise (Error (Undefined_primitive name))

let report_error ppf = function
  | Undefined_class (s, None) ->
      Format.fprintf ppf "Class '%s' cannot be found in classpath" s
  | Undefined_class (s, (Some v)) ->
      Format.fprintf ppf "Class '%s' cannot be loaded: %s" s v
  | Undefined_primitive s ->
      Format.fprintf ppf "Primitive '%s' cannot be found" s
