(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2010 Xavier Clerc.
 * Original file (asmcomp/asmlink.ml in the Objective Caml source
 * distribution) is Copyright (C) INRIA.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://qt.nokia.com/doc/4.0/qpl.html>.
 *)

(** Link of .cmj/.jo files into .jar and .cmjs files. *)

type error =
  | File_not_found of string
  | Not_an_object_file of string
  | Missing_implementations of (string * string list) list
  | Inconsistent_interface of string * string * string
  | Inconsistent_implementation of string * string * string
  | Linking_error
  | Multiple_definition of string * string * string
  | Missing_cmj of string * string
  | Unable_to_add_class of string
  | Unable_to_add_file of string
  | Duplicate_entry of string

exception Error of error


val check_consistency : string -> Jcompilenv.unit_infos -> Digest.t -> unit
(** [check_consistencyfilename unit crc] checks the consistency of [filename]
   declaring [unit] and whose digest is [crc]. *)

val extract_crc_interfaces : unit -> (string * Digest.t) list
(** Returns the list of unitname/digest couples corresponding to the unit
   interface whose consistency has been checked. *)

val extract_crc_implementations : unit -> (string * Digest.t) list
(** Returns the list of unitname/digest couples corresponding to the unit
   implementation whose consistency has been checked. *)

val link : string list -> string -> unit
(** [link files target] constructs the executable [target] from [files]. *)

val link_shared : string list -> string -> unit
(** [link_shared fmt files target] constructs the plugin [target] from [files]. *)

val report_error : Format.formatter -> error -> unit
(** Pretty-prints an error. *)
